@@ -44,13 +44,16 @@ static const char * const license =
 #include <iostream>
 #include <iterator>
 #include <limits>
+#include <locale>
 #include <map>
 #include <random>
 #include <set>
 #include <sstream>
+#include <stdexcept>
 #include <string>
 #include <vector>
 
+#include <cassert>
 #include <cmath>
 #include <cstdint>
 #include <cstdio>
@@ -60,6 +63,7 @@ static const char * const license =
 
 #if defined(__DJGPP__)
 #include <conio.h>
+#include <dpmi.h>
 #include <fcntl.h>
 #include <io.h>
 #include <sys/stat.h>
@@ -77,9 +81,6 @@ static const char * const license =
 #include <mmsystem.h>
 #include <mmreg.h>
 #else
-#if defined(MPT_NEEDS_THREADS)
-#include <pthread.h>
-#endif
 #include <termios.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -100,7 +101,6 @@ static const char * const license =
 #include "openmpt123_allegro42.hpp"
 #include "openmpt123_portaudio.hpp"
 #include "openmpt123_pulseaudio.hpp"
-#include "openmpt123_sdl.hpp"
 #include "openmpt123_sdl2.hpp"
 #include "openmpt123_waveout.hpp"
 
@@ -130,12 +130,33 @@ struct show_version_number_exception : public std::exception {
 struct show_long_version_number_exception : public std::exception {
 };
 
+#if defined( WIN32 )
+bool IsConsole( DWORD stdHandle ) {
+	HANDLE hStd = GetStdHandle( stdHandle );
+	if ( ( hStd != NULL ) && ( hStd != INVALID_HANDLE_VALUE ) ) {
+		DWORD mode = 0;
+		if ( GetConsoleMode( hStd, &mode ) != FALSE ) {
+			return true;
+		}
+	}
+	return false;
+}
+#endif
+
 bool IsTerminal( int fd ) {
 #if defined( WIN32 )
-	return true
-		&& ( _isatty( fd ) ? true : false )
-		&& GetConsoleWindow() != NULL
-		;
+	if ( !_isatty( fd ) ) {
+		return false;
+	}
+	DWORD stdHandle = 0;
+	if ( fd == 0 ) {
+		stdHandle = STD_INPUT_HANDLE;
+	} else if ( fd == 1 ) {
+		stdHandle = STD_OUTPUT_HANDLE;
+	} else if ( fd == 2 ) {
+		stdHandle = STD_ERROR_HANDLE;
+	}
+	return IsConsole( stdHandle );
 #else
 	return isatty( fd ) ? true : false;
 #endif
@@ -167,10 +188,10 @@ static void set_input_mode() {
 
 class file_audio_stream_raii : public file_audio_stream_base {
 private:
-	file_audio_stream_base * impl;
+	std::unique_ptr<file_audio_stream_base> impl;
 public:
 	file_audio_stream_raii( const commandlineflags & flags, const std::string & filename, std::ostream & log )
-		: impl(0)
+		: impl(nullptr)
 	{
 		if ( !flags.force_overwrite ) {
 			std::ifstream testfile( filename, std::ios::binary );
@@ -181,18 +202,18 @@ public:
 		if ( false ) {
 			// nothing
 		} else if ( flags.output_extension == "raw" ) {
-			impl = new raw_stream_raii( filename, flags, log );
+			impl = std::make_unique<raw_stream_raii>( filename, flags, log );
 #ifdef MPT_WITH_MMIO
 		} else if ( flags.output_extension == "wav" ) {
-			impl = new mmio_stream_raii( filename, flags, log );
+			impl = std::make_unique<mmio_stream_raii>( filename, flags, log );
 #endif				
 #ifdef MPT_WITH_FLAC
 		} else if ( flags.output_extension == "flac" ) {
-			impl = new flac_stream_raii( filename, flags, log );
+			impl = std::make_unique<flac_stream_raii>( filename, flags, log );
 #endif				
 #ifdef MPT_WITH_SNDFILE
 		} else {
-			impl = new sndfile_stream_raii( filename, flags, log );
+			impl = std::make_unique<sndfile_stream_raii>( filename, flags, log );
 #endif
 		}
 		if ( !impl ) {
@@ -200,10 +221,7 @@ public:
 		}
 	}
 	virtual ~file_audio_stream_raii() {
-		if ( impl ) {
-			delete impl;
-			impl = 0;
-		}
+		return;
 	}
 	void write_metadata( std::map<std::string,std::string> metadata ) override {
 		impl->write_metadata( metadata );
@@ -497,18 +515,6 @@ static void show_info( std::ostream & log, bool verbose ) {
 	log << "API: " << static_cast<int>( sdlver.major ) << "." << static_cast<int>( sdlver.minor ) << "." << static_cast<int>( sdlver.patch ) << "";
 	log << " <https://libsdl.org/>" << std::endl;
 #endif
-#ifdef MPT_WITH_SDL
-	const SDL_version * linked_sdlver = SDL_Linked_Version();
-	log << " libSDL ";
-	if ( linked_sdlver ) {
-		log << static_cast<int>( linked_sdlver->major ) << "." << static_cast<int>( linked_sdlver->minor ) << "." << static_cast<int>( linked_sdlver->patch ) << " ";
-	}
-	SDL_version sdlver;
-	std::memset( &sdlver, 0, sizeof( SDL_version ) );
-	SDL_VERSION( &sdlver );
-	log << "(API: " << static_cast<int>( sdlver.major ) << "." << static_cast<int>( sdlver.minor ) << "." << static_cast<int>( sdlver.patch ) << ")";
-	log << " <https://libsdl.org/>" << std::endl;
-#endif
 #ifdef MPT_WITH_PULSEAUDIO
 	log << " " << "libpulse, libpulse-simple" << " (headers " << pa_get_headers_version()  << ", API " << PA_API_VERSION << ", PROTOCOL " << PA_PROTOCOL_VERSION << ", library " << ( pa_get_library_version() ? pa_get_library_version() : "unknown" ) << ") <https://www.freedesktop.org/wiki/Software/PulseAudio/>" << std::endl;
 #endif
@@ -712,33 +718,6 @@ static void show_help_keyboard( textout & log ) {
 }
 
 
-template < typename T, typename Tmod >
-T ctl_get( Tmod & mod, const std::string & ctl ) {
-	T result = T();
-	try {
-		std::istringstream str;
-		str.imbue( std::locale::classic() );
-		str.str( mod.ctl_get( ctl ) );
-		str >> std::fixed >> std::setprecision(16) >> result;
-	} catch ( const openmpt::exception & ) {
-		// ignore
-	}
-	return result;
-}
-
-template < typename T, typename Tmod >
-void ctl_set( Tmod & mod, const std::string & ctl, const T & val ) {
-	try {
-		std::ostringstream str;
-		str.imbue( std::locale::classic() );
-		str << std::fixed << std::setprecision(16) << val;
-		mod.ctl_set( ctl, str.str() );
-	} catch ( const openmpt::exception & ) {
-		// ignore
-	}
-	return;
-}
-
 template < typename Tmod >
 static void apply_mod_settings( commandlineflags & flags, Tmod & mod ) {
 	flags.separation = std::max( flags.separation, std::int32_t(   0 ) );
@@ -754,12 +733,17 @@ static void apply_mod_settings( commandlineflags & flags, Tmod & mod ) {
 	mod.set_render_param( openmpt::module::RENDER_STEREOSEPARATION_PERCENT, flags.separation );
 	mod.set_render_param( openmpt::module::RENDER_INTERPOLATIONFILTER_LENGTH, flags.filtertaps );
 	mod.set_render_param( openmpt::module::RENDER_VOLUMERAMPING_STRENGTH, flags.ramping );
-	ctl_set( mod, "play.tempo_factor", tempo_flag_to_double( flags.tempo ) );
-	ctl_set( mod, "play.pitch_factor", pitch_flag_to_double( flags.pitch ) );
-	std::ostringstream dither_str;
-	dither_str.imbue( std::locale::classic() );
-	dither_str << flags.dither;
-	mod.ctl_set( "dither", dither_str.str() );
+	try {
+		mod.ctl_set_floatingpoint( "play.tempo_factor", tempo_flag_to_double( flags.tempo ) );
+	} catch ( const openmpt::exception & ) {
+		// ignore
+	}
+	try {
+		mod.ctl_set_floatingpoint( "play.pitch_factor", pitch_flag_to_double( flags.pitch ) );
+	} catch ( const openmpt::exception & ) {
+		// ignore
+	}
+	mod.ctl_set_integer( "dither", flags.dither );
 }
 
 struct prev_file { int count; prev_file( int c ) : count(c) { } };
@@ -1037,9 +1021,9 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 	log.writeout();
 
 	std::size_t bufsize;
-	if ( flags.mode == ModeUI ) {
+	if ( flags.mode == Mode::UI ) {
 		bufsize = std::min( flags.ui_redraw_interval, flags.period ) * flags.samplerate / 1000;
-	} else if ( flags.mode == ModeBatch ) {
+	} else if ( flags.mode == Mode::Batch ) {
 		bufsize = flags.period * flags.samplerate / 1000;
 	} else {
 		bufsize = 1024;
@@ -1102,28 +1086,11 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 
 	log.writeout();
 
-#if defined( WIN32 )
-	HANDLE hStdErr = NULL;
-	COORD coord_cursor = COORD();
-	if ( multiline ) {
-		log.flush();
-		hStdErr = GetStdHandle( STD_ERROR_HANDLE );
-		if ( hStdErr ) {
-			CONSOLE_SCREEN_BUFFER_INFO csbi;
-			ZeroMemory( &csbi, sizeof( CONSOLE_SCREEN_BUFFER_INFO ) );
-			GetConsoleScreenBufferInfo( hStdErr, &csbi );
-			coord_cursor = csbi.dwCursorPosition;
-			coord_cursor.X = 1;
-			coord_cursor.Y -= lines;
-		}
-	}
-#endif
-
 	double cpu_smooth = 0.0;
 
 	while ( true ) {
 
-		if ( flags.mode == ModeUI ) {
+		if ( flags.mode == Mode::UI ) {
 
 #if defined( __DJGPP__ )
 
@@ -1134,6 +1101,15 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 				}
 			}
 
+#elif defined( WIN32 ) && defined( UNICODE )
+
+			while ( _kbhit() ) {
+				wint_t c = _getwch();
+				if ( !handle_keypress( c, flags, mod, audio_stream ) ) {
+					return;
+				}
+			}
+
 #elif defined( WIN32 )
 
 			while ( _kbhit() ) {
@@ -1216,16 +1192,7 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 		}
 
 		if ( multiline ) {
-#if defined( WIN32 )
-			log.flush();
-			if ( hStdErr ) {
-				SetConsoleCursorPosition( hStdErr, coord_cursor );
-			}
-#else
-			for ( int line = 0; line < lines; ++line ) {
-				log << "\x1b[1A";
-			}
-#endif
+			log.cursor_up( lines );
 			log << std::endl;
 			if ( flags.show_meters ) {
 				draw_meters( log, meter, flags );
@@ -1507,7 +1474,7 @@ void render_mod_file( commandlineflags & flags, const std::string & filename, st
 
 	log.writeout();
 
-	if ( flags.mode != ModeProbe && flags.mode != ModeInfo ) {
+	if ( flags.mode != Mode::Probe && flags.mode != Mode::Info ) {
 		mod.set_repeat_count( flags.repeatcount );
 		apply_mod_settings( flags, mod );
 	}
@@ -1577,13 +1544,13 @@ void render_mod_file( commandlineflags & flags, const std::string & filename, st
 
 	log.writeout();
 
-	if ( flags.filenames.size() == 1 || flags.mode == ModeRender ) {
+	if ( flags.filenames.size() == 1 || flags.mode == Mode::Render ) {
 		audio_stream.write_metadata( get_metadata( mod ) );
 	} else {
 		audio_stream.write_updated_metadata( get_metadata( mod ) );
 	}
 
-	if ( flags.mode == ModeProbe || flags.mode == ModeInfo ) {
+	if ( flags.mode == Mode::Probe || flags.mode == Mode::Info ) {
 		return;
 	}
 
@@ -2019,15 +1986,15 @@ static commandlineflags parse_openmpt123( const std::vector<std::string> & args,
 			} else if ( arg == "--license" ) {
 				throw show_license_exception();
 			} else if ( arg == "--probe" ) {
-				flags.mode = ModeProbe;
+				flags.mode = Mode::Probe;
 			} else if ( arg == "--info" ) {
-				flags.mode = ModeInfo;
+				flags.mode = Mode::Info;
 			} else if ( arg == "--ui" ) {
-				flags.mode = ModeUI;
+				flags.mode = Mode::UI;
 			} else if ( arg == "--batch" ) {
-				flags.mode = ModeBatch;
+				flags.mode = Mode::Batch;
 			} else if ( arg == "--render" ) {
-				flags.mode = ModeRender;
+				flags.mode = Mode::Render;
 			} else if ( arg == "--terminal-width" && nextarg != "" ) {
 				std::istringstream istr( nextarg );
 				istr >> flags.terminal_width;
@@ -2073,9 +2040,6 @@ static commandlineflags parse_openmpt123( const std::vector<std::string> & args,
 #if defined( MPT_WITH_SDL2 )
 					drivers << "    " << "sdl2" << std::endl;
 #endif
-#if defined( MPT_WITH_SDL )
-					drivers << "    " << "sdl" << std::endl;
-#endif
 #if defined( MPT_WITH_PORTAUDIO )
 					drivers << "    " << "portaudio" << std::endl;
 #endif
@@ -2246,21 +2210,33 @@ static commandlineflags parse_openmpt123( const std::vector<std::string> & args,
 
 #if defined(WIN32)
 
-class ConsoleCP_utf8_raii {
+class FD_utf8_raii {
 private:
-	const UINT oldCP;
-	const UINT oldOutputCP;
+	FILE * file;
+	int old_mode;
 public:
-	ConsoleCP_utf8_raii()
-		: oldCP(GetConsoleCP())
-		, oldOutputCP(GetConsoleOutputCP())
+	FD_utf8_raii( FILE * file, bool set_utf8 )
+		: file(file)
+		, old_mode(-1)
 	{
-		SetConsoleCP( 65001 ); // UTF-8
-		SetConsoleOutputCP( 65001 ); // UTF-8
+		if ( set_utf8 ) {
+			fflush( file );
+			#if defined(UNICODE)
+				old_mode = _setmode( _fileno( file ), _O_U8TEXT );
+			#else
+				old_mode = _setmode( _fileno( file ), _O_TEXT );
+			#endif
+			if ( old_mode == -1 ) {
+				throw exception( "failed to set TEXT mode on file descriptor" );
+			}
+		}
 	}
-	~ConsoleCP_utf8_raii() {
-		SetConsoleCP( oldCP );
-		SetConsoleOutputCP( oldOutputCP );
+	~FD_utf8_raii()
+	{
+		if ( old_mode != -1 ) {
+			fflush( file );
+			old_mode = _setmode( _fileno( file ), old_mode );
+		}
 	}
 };
 
@@ -2269,7 +2245,7 @@ private:
 	FILE * file;
 	int old_mode;
 public:
-	FD_binary_raii(FILE * file, bool set_binary)
+	FD_binary_raii( FILE * file, bool set_binary )
 		: file(file)
 		, old_mode(-1)
 	{
@@ -2307,12 +2283,19 @@ static int main( int argc, char * argv [] ) {
 	#endif
 
 #if defined(WIN32)
-	ConsoleCP_utf8_raii console_cp;
+	FD_utf8_raii stdin_utf8_guard( stdin, true );
+	FD_utf8_raii stdout_utf8_guard( stdout, true );
+	FD_utf8_raii stderr_utf8_guard( stderr, true );
 #endif
 	textout_dummy dummy_log;
 #if defined(WIN32)
-	textout_console std_out( GetStdHandle( STD_OUTPUT_HANDLE ) );
-	textout_console std_err( GetStdHandle( STD_ERROR_HANDLE ) );
+#if defined(UNICODE)
+	textout_ostream_console std_out( std::wcout, STD_OUTPUT_HANDLE );
+	textout_ostream_console std_err( std::wclog, STD_ERROR_HANDLE );
+#else
+	textout_ostream_console std_out( std::cout, STD_OUTPUT_HANDLE );
+	textout_ostream_console std_err( std::clog, STD_ERROR_HANDLE );
+#endif
 #else
 	textout_ostream std_out( std::cout );
 	textout_ostream std_err( std::clog );
@@ -2399,16 +2382,16 @@ static int main( int argc, char * argv [] ) {
 		// setup terminal
 		#if !defined(WIN32)
 			if ( stdin_can_ui ) {
-				if ( flags.mode == ModeUI ) {
+				if ( flags.mode == Mode::UI ) {
 					set_input_mode();
 				}
 			}
 		#endif
 		
-		textout & log = flags.quiet ? *static_cast<textout*>( &dummy_log ) : *static_cast<textout*>( stdout_can_ui ? &std_out : &std_err );
+		textout & log = flags.quiet ? static_cast<textout&>( dummy_log ) : static_cast<textout&>( stdout_can_ui ? std_out : std_err );
 
 		show_info( log, flags.verbose );
-		
+
 		if ( !flags.warnings.empty() ) {
 			log << flags.warnings << std::endl;
 		}
@@ -2431,18 +2414,18 @@ static int main( int argc, char * argv [] ) {
 		std::srand( std::uniform_int_distribution<unsigned int>()( prng ) );
 
 		switch ( flags.mode ) {
-			case ModeProbe: {
+			case Mode::Probe: {
 				for ( const auto & filename : flags.filenames ) {
 					probe_file( flags, filename, log );
 					flags.playlist_index++;
 				}
 			} break;
-			case ModeInfo: {
+			case Mode::Info: {
 				void_audio_stream dummy;
 				render_files( flags, log, dummy, prng );
 			} break;
-			case ModeUI:
-			case ModeBatch: {
+			case Mode::UI:
+			case Mode::Batch: {
 				if ( flags.use_stdout ) {
 					flags.apply_default_buffer_sizes();
 					stdout_stream_raii stdout_audio_stream;
@@ -2461,11 +2444,6 @@ static int main( int argc, char * argv [] ) {
 					sdl2_stream_raii sdl2_stream( flags, log );
 					render_files( flags, log, sdl2_stream, prng );
 #endif
-#if defined( MPT_WITH_SDL )
-				} else if ( flags.driver == "sdl" || flags.driver.empty() ) {
-					sdl_stream_raii sdl_stream( flags, log );
-					render_files( flags, log, sdl_stream, prng );
-#endif
 #if defined( MPT_WITH_PORTAUDIO )
 				} else if ( flags.driver == "portaudio" || flags.driver.empty() ) {
 					portaudio_stream_raii portaudio_stream( flags, log );
@@ -2489,7 +2467,7 @@ static int main( int argc, char * argv [] ) {
 					}
 				}
 			} break;
-			case ModeRender: {
+			case Mode::Render: {
 				for ( const auto & filename : flags.filenames ) {
 					flags.apply_default_buffer_sizes();
 					file_audio_stream_raii file_audio_stream( flags, filename + std::string(".") + flags.output_extension, log );
@@ -2497,7 +2475,7 @@ static int main( int argc, char * argv [] ) {
 					flags.playlist_index++;
 				}
 			} break;
-			case ModeNone:
+			case Mode::None:
 			break;
 		}
 
@@ -2522,12 +2500,6 @@ static int main( int argc, char * argv [] ) {
 		std_err.writeout();
 		return 1;
 #endif
-#ifdef MPT_WITH_SDL
-	} catch ( sdl_exception & e ) {
-		std_err << "SDL error: " << e.what() << std::endl;
-		std_err.writeout();
-		return 1;
-#endif
 #ifdef MPT_WITH_SDL2
 	} catch ( sdl2_exception & e ) {
 		std_err << "SDL2 error: " << e.what() << std::endl;
