@@ -21,16 +21,31 @@
 
 #include "MPEGFrame.h"
 #include "OggStream.h"
+
 #if defined(MPT_WITH_VORBIS) && defined(MPT_WITH_VORBISFILE)
-#include "../common/mptBufferIO.h"
+#include <sstream>
 #endif
 
 #if defined(MPT_WITH_VORBIS)
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
+#endif  // MPT_COMPILER_CLANG
 #include <vorbis/codec.h>
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic pop
+#endif  // MPT_COMPILER_CLANG
 #endif
 
 #if defined(MPT_WITH_VORBISFILE)
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wreserved-id-macro"
+#endif  // MPT_COMPILER_CLANG
 #include <vorbis/vorbisfile.h>
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic pop
+#endif  // MPT_COMPILER_CLANG
 #include "../soundbase/SampleFormatConverters.h"
 #include "../soundbase/SampleFormatCopy.h"
 #endif
@@ -39,7 +54,7 @@
 #include <stb_vorbis/stb_vorbis.c>
 #include "../soundbase/SampleFormatConverters.h"
 #include "../soundbase/SampleFormatCopy.h"
-#endif // MPT_WITH_STBVORBIS
+#endif  // MPT_WITH_STBVORBIS
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -63,7 +78,7 @@ struct MO3FileHeader
 		itCompatGxx		= 0x0400,
 		itOldFX			= 0x0800,
 		modplugMode		= 0x10000,
-		unknown			= 0x20000,	// Always set
+		unknown			= 0x20000,	// Always set (internal BASS flag to designate modules)
 		modVBlank		= 0x80000,
 		hasPlugins		= 0x100000,
 		extFilterRange	= 0x200000,
@@ -118,7 +133,7 @@ struct MO3Envelope
 		if(flags & envLoop) mptEnv.dwFlags.set(ENV_LOOP);
 		if(flags & envFilter) mptEnv.dwFlags.set(ENV_FILTER);
 		if(flags & envCarry) mptEnv.dwFlags.set(ENV_CARRY);
-		mptEnv.resize(std::min<uint8>(numNodes, 25));
+		mptEnv.resize(std::min(numNodes.get(), uint8(25)));
 		mptEnv.nSustainStart = sustainStart;
 		mptEnv.nSustainEnd = sustainEnd;
 		mptEnv.nLoopStart = loopStart;
@@ -224,7 +239,7 @@ struct MO3Instrument
 		}
 
 		if(type == MOD_TYPE_IT)
-			mptIns.nGlobalVol = std::min<uint8>(globalVol, 128) / 2u;
+			mptIns.nGlobalVol = std::min(static_cast<uint8>(globalVol), uint8(128)) / 2u;
 		if(panning <= 256)
 		{
 			mptIns.nPan = panning;
@@ -235,8 +250,8 @@ struct MO3Instrument
 		mptIns.nPPC = ppc;
 		mptIns.nDCT = static_cast<DuplicateCheckType>(dct.get());
 		mptIns.nDNA = static_cast<DuplicateNoteAction>(dca.get());
-		mptIns.nVolSwing = static_cast<uint8>(std::min<uint16>(volSwing, 100));
-		mptIns.nPanSwing = static_cast<uint8>(std::min<uint16>(panSwing, 256) / 4u);
+		mptIns.nVolSwing = static_cast<uint8>(std::min(volSwing.get(), uint16(100)));
+		mptIns.nPanSwing = static_cast<uint8>(std::min(panSwing.get(), uint16(256)) / 4u);
 		mptIns.SetCutoff(cutoff & 0x7F, (cutoff & 0x80) != 0);
 		mptIns.SetResonance(resonance & 0x7F, (resonance & 0x80) != 0);
 	}
@@ -295,10 +310,11 @@ struct MO3Sample
 		} else
 		{
 			mptSmp.nFineTune = static_cast<int8>(freqFinetune);
-			if(type != MOD_TYPE_MTM) mptSmp.nFineTune -= 128;
+			if(type != MOD_TYPE_MTM)
+				mptSmp.nFineTune -= 128;
 			mptSmp.RelativeTone = transpose;
 		}
-		mptSmp.nVolume = std::min<uint8>(defaultVolume, 64) * 4u;
+		mptSmp.nVolume = std::min(defaultVolume.get(), uint8(64)) * 4u;
 		if(panning <= 256)
 		{
 			mptSmp.nPan = panning;
@@ -307,10 +323,14 @@ struct MO3Sample
 		mptSmp.nLength = length;
 		mptSmp.nLoopStart = loopStart;
 		mptSmp.nLoopEnd = loopEnd;
-		if(flags & smpLoop) mptSmp.uFlags.set(CHN_LOOP);
-		if(flags & smpPingPongLoop) mptSmp.uFlags.set(CHN_PINGPONGLOOP);
-		if(flags & smpSustain) mptSmp.uFlags.set(CHN_SUSTAINLOOP);
-		if(flags & smpSustainPingPong) mptSmp.uFlags.set(CHN_PINGPONGSUSTAIN);
+		if(flags & smpLoop)
+			mptSmp.uFlags.set(CHN_LOOP);
+		if(flags & smpPingPongLoop)
+			mptSmp.uFlags.set(CHN_PINGPONGLOOP);
+		if(flags & smpSustain)
+			mptSmp.uFlags.set(CHN_SUSTAINLOOP);
+		if(flags & smpSustainPingPong)
+			mptSmp.uFlags.set(CHN_PINGPONGSUSTAIN);
 
 		mptSmp.nVibType = static_cast<VibratoType>(AutoVibratoIT2XM[vibType & 7]);
 		mptSmp.nVibSweep = vibSweep;
@@ -318,7 +338,7 @@ struct MO3Sample
 		mptSmp.nVibRate = vibRate;
 
 		if(type == MOD_TYPE_IT)
-			mptSmp.nGlobalVol = std::min<uint8>(globalVol, 64);
+			mptSmp.nGlobalVol = std::min(static_cast<uint8>(globalVol), uint8(64));
 		mptSmp.nSustainStart = sustainStart;
 		mptSmp.nSustainEnd = sustainEnd;
 	}
@@ -336,7 +356,7 @@ struct MO3SampleChunk
 	uint16 headerSize;
 	int16 sharedHeader;
 	MO3SampleChunk(const FileReader &chunk_ = FileReader(), uint16 headerSize_ = 0, int16 sharedHeader_ = 0)
-		: chunk(chunk_), headerSize(headerSize_), sharedHeader(sharedHeader_) { }
+	    : chunk(chunk_), headerSize(headerSize_), sharedHeader(sharedHeader_) {}
 };
 
 
@@ -354,7 +374,10 @@ struct MO3SampleChunk
 	data &= 0xFF; \
 	if(data == 0) \
 	{ \
-		data = file.ReadUint8(); \
+		uint8 nextByte; \
+		if(!file.Read(nextByte)) \
+			break; \
+		data = nextByte; \
 		data = (data << 1) + 1; \
 		carry = (data > 0xFF); \
 		data &= 0xFF; \
@@ -366,14 +389,15 @@ struct MO3SampleChunk
 // until second bit is 0 (noted n0)
 
 #define DECODE_CTRL_BITS \
-{ \
-	strLen++; \
-	do { \
-		READ_CTRL_BIT; \
-		strLen = (strLen << 1) + carry; \
-		READ_CTRL_BIT; \
-	} while(carry); \
-}
+	{ \
+		strLen++; \
+		do \
+		{ \
+			READ_CTRL_BIT; \
+			strLen = (strLen << 1) + carry; \
+			READ_CTRL_BIT; \
+		} while(carry); \
+	}
 
 
 static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, const uint32 size)
@@ -382,9 +406,9 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 		return false;
 
 	uint16 data = 0;
-	int8 carry = 0;			// x86 carry (used to propagate the most significant bit from one byte to another)
-	int32 strLen = 0;		// length of previous string
-	int32 strOffset;		// string offset
+	int8 carry = 0;    // x86 carry (used to propagate the most significant bit from one byte to another)
+	int32 strLen = 0;  // length of previous string
+	int32 strOffset;   // string offset
 	uint32 previousPtr = 0;
 
 	// Read first uncompressed byte
@@ -397,9 +421,10 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 		if(!carry)
 		{
 			// a 0 ctrl bit means 'copy', not compressed byte
-			if(!file.CanRead(1))
+			if(uint8 b; file.Read(b))
+				uncompressed.push_back(b);
+			else
 				break;
-			uncompressed.push_back(file.ReadUint8());
 			remain--;
 		} else
 		{
@@ -410,12 +435,15 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 			if(strLen < 0)
 			{
 				// means LZ ptr with same previous relative LZ ptr (saved one)
-				strOffset = previousPtr;	// restore previous Ptr
+				strOffset = previousPtr;  // restore previous Ptr
 				strLen++;
 			} else
 			{
 				// LZ ptr in ctrl stream
-				strOffset = (strLen << 8) | file.ReadUint8(); // read less significant offset byte from stream
+				if(uint8 b; file.Read(b))
+					strOffset = (strLen << 8) | b;  // read less significant offset byte from stream
+				else
+					break;
 				strLen = 0;
 				strOffset = ~strOffset;
 				if(strOffset < -1280)
@@ -423,7 +451,7 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 				lengthAdjust++;  // length is always at least 1
 				if(strOffset < -32000)
 					lengthAdjust++;
-				previousPtr = strOffset; // save current Ptr
+				previousPtr = strOffset;  // save current Ptr
 			}
 
 			// read the next 2 bits as part of strLen
@@ -434,8 +462,8 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 			if(strLen == 0)
 			{
 				// length does not fit in 2 bits
-				DECODE_CTRL_BITS;	// decode length: 1 is the most significant bit,
-				strLen += 2;		// then first bit of each bits pairs (noted n1), until n0.
+				DECODE_CTRL_BITS;  // decode length: 1 is the most significant bit,
+				strLen += 2;       // then first bit of each bits pairs (noted n1), until n0.
 			}
 			strLen += lengthAdjust;  // length adjustment
 
@@ -469,12 +497,12 @@ static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, co
 
 struct MO3Delta8BitParams
 {
-	typedef int8 sample_t;
-	typedef uint8 unsigned_t;
-	enum : int { shift = 7 };
-	enum : uint8 { dhInit = 4 };
+	using sample_t = int8;
+	using unsigned_t = uint8;
+	static constexpr int shift = 7;
+	static constexpr uint8 dhInit = 4;
 
-	static inline void Decode(FileReader &file, int8 &carry, uint16 &data, uint8 &/*dh*/, unsigned_t &val)
+	static inline void Decode(FileReader &file, int8 &carry, uint16 &data, uint8 & /*dh*/, unsigned_t &val)
 	{
 		do
 		{
@@ -487,10 +515,10 @@ struct MO3Delta8BitParams
 
 struct MO3Delta16BitParams
 {
-	typedef int16 sample_t;
-	typedef uint16 unsigned_t;
-	enum : int { shift = 15 };
-	enum : uint8 { dhInit = 8 };
+	using sample_t = int16;
+	using unsigned_t = uint16;
+	static constexpr int shift = 15;
+	static constexpr uint8 dhInit = 8;
 
 	static inline void Decode(FileReader &file, int8 &carry, uint16 &data, uint8 &dh, unsigned_t &val)
 	{
@@ -502,7 +530,7 @@ struct MO3Delta16BitParams
 				val = (val << 1) + carry;
 				READ_CTRL_BIT;
 				val = (val << 1) + carry;
-				READ_CTRL_BIT; \
+				READ_CTRL_BIT;
 			} while(carry);
 		} else
 		{
@@ -517,7 +545,7 @@ struct MO3Delta16BitParams
 };
 
 
-template<typename Properties>
+template <typename Properties>
 static void UnpackMO3DeltaSample(FileReader &file, typename Properties::sample_t *dst, uint32 length, uint8 numChannels)
 {
 	uint8 dh = Properties::dhInit, cl = 0;
@@ -529,7 +557,7 @@ static void UnpackMO3DeltaSample(FileReader &file, typename Properties::sample_t
 	for(uint8 chn = 0; chn < numChannels; chn++)
 	{
 		typename Properties::sample_t *p = dst + chn;
-		const typename Properties::sample_t * const pEnd = p + length * numChannels;
+		const typename Properties::sample_t *const pEnd = p + length * numChannels;
 		while(p < pEnd)
 		{
 			val = 0;
@@ -549,12 +577,12 @@ static void UnpackMO3DeltaSample(FileReader &file, typename Properties::sample_t
 					cl--;
 			}
 			dh = dh + cl;
-			dh >>= 1;			// next length in bits of encoded delta second part
-			carry = val & 1;	// sign of delta 1=+, 0=not
+			dh >>= 1;         // next length in bits of encoded delta second part
+			carry = val & 1;  // sign of delta 1=+, 0=not
 			val >>= 1;
 			if(carry == 0)
-				val = ~val;		// negative delta
-			val += previous;	// previous value + delta
+				val = ~val;   // negative delta
+			val += previous;  // previous value + delta
 			*p = val;
 			p += numChannels;
 			previous = val;
@@ -563,7 +591,7 @@ static void UnpackMO3DeltaSample(FileReader &file, typename Properties::sample_t
 }
 
 
-template<typename Properties>
+template <typename Properties>
 static void UnpackMO3DeltaPredictionSample(FileReader &file, typename Properties::sample_t *dst, uint32 length, uint8 numChannels)
 {
 	uint8 dh = Properties::dhInit, cl = 0;
@@ -576,12 +604,12 @@ static void UnpackMO3DeltaPredictionSample(FileReader &file, typename Properties
 	for(uint8 chn = 0; chn < numChannels; chn++)
 	{
 		typename Properties::sample_t *p = dst + chn;
-		const typename Properties::sample_t * const pEnd = p + length * numChannels;
+		const typename Properties::sample_t *const pEnd = p + length * numChannels;
 		while(p < pEnd)
 		{
 			val = 0;
 			Properties::Decode(file, carry, data, dh, val);
-			cl = dh;	// length in bits of: delta second part (right most bits of delta) and sign bit
+			cl = dh;  // length in bits of: delta second part (right most bits of delta) and sign bit
 			while(cl > 0)
 			{
 				READ_CTRL_BIT;
@@ -596,18 +624,18 @@ static void UnpackMO3DeltaPredictionSample(FileReader &file, typename Properties
 					cl--;
 			}
 			dh = dh + cl;
-			dh >>= 1;			// next length in bits of encoded delta second part
-			carry = val & 1;	// sign of delta 1=+, 0=not
+			dh >>= 1;         // next length in bits of encoded delta second part
+			carry = val & 1;  // sign of delta 1=+, 0=not
 			val >>= 1;
 			if(carry == 0)
-				val = ~val;		// negative delta
+				val = ~val;  // negative delta
 
 			delta = static_cast<typename Properties::sample_t>(val);
-			val = val + static_cast<typename Properties::unsigned_t>(next);	// predicted value + delta
+			val = val + static_cast<typename Properties::unsigned_t>(next);  // predicted value + delta
 			*p = val;
 			p += numChannels;
 			sval = static_cast<typename Properties::sample_t>(val);
-			next = (sval * (1<<1)) + (delta >> 1) - previous;  // corrected next value
+			next = (sval * (1 << 1)) + (delta >> 1) - previous;  // corrected next value
 
 			Limit(next, std::numeric_limits<typename Properties::sample_t>::min(), std::numeric_limits<typename Properties::sample_t>::max());
 
@@ -625,60 +653,55 @@ static void UnpackMO3DeltaPredictionSample(FileReader &file, typename Properties
 
 static size_t VorbisfileFilereaderRead(void *ptr, size_t size, size_t nmemb, void *datasource)
 {
-	FileReader &file = *reinterpret_cast<FileReader*>(datasource);
-	return file.ReadRaw(mpt::void_cast<mpt::byte*>(ptr), size * nmemb) / size;
+	FileReader &file = *reinterpret_cast<FileReader *>(datasource);
+	return file.ReadRaw(mpt::void_cast<std::byte *>(ptr), size * nmemb) / size;
 }
 
 static int VorbisfileFilereaderSeek(void *datasource, ogg_int64_t offset, int whence)
 {
-	FileReader &file = *reinterpret_cast<FileReader*>(datasource);
+	FileReader &file = *reinterpret_cast<FileReader *>(datasource);
 	switch(whence)
 	{
 	case SEEK_SET:
+		if(!Util::TypeCanHoldValue<FileReader::off_t>(offset))
 		{
-			if(!Util::TypeCanHoldValue<FileReader::off_t>(offset))
-			{
-				return -1;
-			}
-			return file.Seek(mpt::saturate_cast<FileReader::off_t>(offset)) ? 0 : -1;
+			return -1;
 		}
-		break;
+		return file.Seek(mpt::saturate_cast<FileReader::off_t>(offset)) ? 0 : -1;
+
 	case SEEK_CUR:
+		if(offset < 0)
 		{
-			if(offset < 0)
+			if(offset == std::numeric_limits<ogg_int64_t>::min())
 			{
-				if(offset == std::numeric_limits<ogg_int64_t>::min())
-				{
-					return -1;
-				}
-				if(!Util::TypeCanHoldValue<FileReader::off_t>(0-offset))
-				{
-					return -1;
-				}
-				return file.SkipBack(mpt::saturate_cast<FileReader::off_t>(0 - offset)) ? 0 : -1;
-			} else
-			{
-				if(!Util::TypeCanHoldValue<FileReader::off_t>(offset))
-				{
-					return -1;
-				}
-				return file.Skip(mpt::saturate_cast<FileReader::off_t>(offset)) ? 0 : -1;
+				return -1;
 			}
-		}
-		break;
-	case SEEK_END:
+			if(!Util::TypeCanHoldValue<FileReader::off_t>(0 - offset))
+			{
+				return -1;
+			}
+			return file.SkipBack(mpt::saturate_cast<FileReader::off_t>(0 - offset)) ? 0 : -1;
+		} else
 		{
 			if(!Util::TypeCanHoldValue<FileReader::off_t>(offset))
 			{
 				return -1;
 			}
-			if(!Util::TypeCanHoldValue<FileReader::off_t>(file.GetLength() + offset))
-			{
-				return -1;
-			}
-			return file.Seek(mpt::saturate_cast<FileReader::off_t>(file.GetLength() + offset)) ? 0 : -1;
+			return file.Skip(mpt::saturate_cast<FileReader::off_t>(offset)) ? 0 : -1;
 		}
 		break;
+
+	case SEEK_END:
+		if(!Util::TypeCanHoldValue<FileReader::off_t>(offset))
+		{
+			return -1;
+		}
+		if(!Util::TypeCanHoldValue<FileReader::off_t>(file.GetLength() + offset))
+		{
+			return -1;
+		}
+		return file.Seek(mpt::saturate_cast<FileReader::off_t>(file.GetLength() + offset)) ? 0 : -1;
+
 	default:
 		return -1;
 	}
@@ -686,7 +709,7 @@ static int VorbisfileFilereaderSeek(void *datasource, ogg_int64_t offset, int wh
 
 static long VorbisfileFilereaderTell(void *datasource)
 {
-	FileReader &file = *reinterpret_cast<FileReader*>(datasource);
+	FileReader &file = *reinterpret_cast<FileReader *>(datasource);
 	FileReader::off_t result = file.GetPosition();
 	if(!Util::TypeCanHoldValue<long>(result))
 	{
@@ -695,7 +718,7 @@ static long VorbisfileFilereaderTell(void *datasource)
 	return static_cast<long>(result);
 }
 
-#endif // MPT_WITH_VORBIS && MPT_WITH_VORBISFILE
+#endif  // MPT_WITH_VORBIS && MPT_WITH_VORBISFILE
 
 
 struct MO3ContainerHeader
@@ -797,9 +820,9 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 
 	MO3FileHeader fileHeader;
 	if(!musicChunk.ReadStruct(fileHeader)
-		|| fileHeader.numChannels == 0 || fileHeader.numChannels > MAX_BASECHANNELS
-		|| fileHeader.numInstruments >= MAX_INSTRUMENTS
-		|| fileHeader.numSamples >= MAX_SAMPLES)
+	   || fileHeader.numChannels == 0 || fileHeader.numChannels > MAX_BASECHANNELS
+	   || fileHeader.numInstruments >= MAX_INSTRUMENTS
+	   || fileHeader.numSamples >= MAX_SAMPLES)
 	{
 		return false;
 	}
@@ -811,35 +834,18 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	m_nDefaultSpeed = fileHeader.defaultSpeed ? fileHeader.defaultSpeed : 6;
 	m_nDefaultTempo.Set(fileHeader.defaultTempo ? fileHeader.defaultTempo : 125, 0);
 
-	mpt::ustring originalFormatType;
-	mpt::ustring originalFormatName;
 	if(fileHeader.flags & MO3FileHeader::isIT)
-	{
 		SetType(MOD_TYPE_IT);
-		originalFormatType = U_("it");
-		originalFormatName = U_("Impulse Tracker");
-	} else if(fileHeader.flags & MO3FileHeader::isS3M)
-	{
+	else if(fileHeader.flags & MO3FileHeader::isS3M)
 		SetType(MOD_TYPE_S3M);
-		originalFormatType = U_("s3m");
-		originalFormatName = U_("ScreamTracker 3");
-	} else if(fileHeader.flags & MO3FileHeader::isMOD)
-	{
+	else if(fileHeader.flags & MO3FileHeader::isMOD)
 		SetType(MOD_TYPE_MOD);
-		originalFormatType = U_("mod");
-		originalFormatName = U_("Generic MOD");
-	} else if(fileHeader.flags & MO3FileHeader::isMTM)
-	{
+	else if(fileHeader.flags & MO3FileHeader::isMTM)
 		SetType(MOD_TYPE_MTM);
-		originalFormatType = U_("mtm");
-		originalFormatName = U_("MultiTracker");
-	} else
-	{
+	else
 		SetType(MOD_TYPE_XM);
-		originalFormatType = U_("xm");
-		originalFormatName = U_("FastTracker 2");
-	}
 
+	m_SongFlags.set(SONG_IMPORTED);
 	if(fileHeader.flags & MO3FileHeader::linearSlides)
 		m_SongFlags.set(SONG_LINEARSLIDES);
 	if((fileHeader.flags & MO3FileHeader::s3mAmigaLimits) && m_nType == MOD_TYPE_S3M)
@@ -856,9 +862,9 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		m_playBehaviour.set(kMODVBlankTiming);
 
 	if(m_nType == MOD_TYPE_IT)
-		m_nDefaultGlobalVolume = std::min<uint16>(fileHeader.globalVol, 128) * 2;
+		m_nDefaultGlobalVolume = std::min(fileHeader.globalVol.get(), uint8(128)) * 2;
 	else if(m_nType == MOD_TYPE_S3M)
-		m_nDefaultGlobalVolume = std::min<uint16>(fileHeader.globalVol, 64) * 4;
+		m_nDefaultGlobalVolume = std::min(fileHeader.globalVol.get(), uint8(64)) * 4;
 
 	if(fileHeader.sampleVolume < 0)
 		m_nSamplePreAmp = fileHeader.sampleVolume + 52;
@@ -870,7 +876,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	for(CHANNELINDEX i = 0; i < headerChannels; i++)
 	{
 		if(m_nType == MOD_TYPE_IT)
-			ChnSettings[i].nVolume = std::min<uint8>(fileHeader.chnVolume[i], 64);
+			ChnSettings[i].nVolume = std::min(fileHeader.chnVolume[i].get(), uint8(64));
 		if(m_nType != MOD_TYPE_XM)
 		{
 			if(fileHeader.chnPan[i] == 127)
@@ -988,7 +994,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	L= 06 00 22 x
 	*/
 
-	static const ModCommand::COMMAND effTrans[] =
+	static constexpr ModCommand::COMMAND effTrans[] =
 	{
 		CMD_NONE,				CMD_NONE,				CMD_NONE,				CMD_ARPEGGIO,
 		CMD_PORTAMENTOUP,		CMD_PORTAMENTODOWN,		CMD_TONEPORTAMENTO,		CMD_VIBRATO,
@@ -1049,11 +1055,16 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 					case 0x01:
 						// Note
 						m.note = cmd[1];
-						if(m.note < 120) m.note += noteOffset;
-						else if(m.note == 0xFF) m.note = NOTE_KEYOFF;
-						else if(m.note == 0xFE) m.note = NOTE_NOTECUT;
-						else m.note = NOTE_FADE;
-						if(!m.IsAmigaNote()) onlyAmigaNotes = false;
+						if(m.note < 120)
+							m.note += noteOffset;
+						else if(m.note == 0xFF)
+							m.note = NOTE_KEYOFF;
+						else if(m.note == 0xFE)
+							m.note = NOTE_NOTECUT;
+						else
+							m.note = NOTE_FADE;
+						if(!m.IsAmigaNote())
+							onlyAmigaNotes = false;
 						break;
 					case 0x02:
 						// Instrument
@@ -1106,7 +1117,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 								break;
 							}
 							if((m_nType == MOD_TYPE_IT && !(cmd[1] & 0x03))
-								|| (m_nType == MOD_TYPE_XM && !(cmd[1] & 0x0F)))
+							   || (m_nType == MOD_TYPE_XM && !(cmd[1] & 0x0F)))
 							{
 								m.volcmd = VOLCMD_PANNING;
 								m.vol = cmd[1] / 4;
@@ -1257,10 +1268,12 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		if(isSampleMode || (pIns = AllocateInstrument(ins)) == nullptr)
 		{
 			// Even in IT sample mode, instrument headers are still stored....
-			while(musicChunk.ReadUint8() != 0);
+			while(musicChunk.ReadUint8() != 0)
+				;
 			if(version >= 5)
 			{
-				while(musicChunk.ReadUint8() != 0);
+				while(musicChunk.ReadUint8() != 0)
+					;
 			}
 			musicChunk.Skip(sizeof(MO3Instrument));
 			continue;
@@ -1268,11 +1281,11 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 
 		std::string name;
 		musicChunk.ReadNullString(name);
-		mpt::String::Copy(pIns->name, name);
+		pIns->name = name;
 		if(version >= 5)
 		{
 			musicChunk.ReadNullString(name);
-			mpt::String::Copy(pIns->filename, name);
+			pIns->filename = name;
 		}
 
 		MO3Instrument insHeader;
@@ -1295,11 +1308,11 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		ModSample &sample = Samples[smp];
 		std::string name;
 		musicChunk.ReadNullString(name);
-		mpt::String::Copy(m_szNames[smp], name);
+		m_szNames[smp] = name;
 		if(version >= 5)
 		{
 			musicChunk.ReadNullString(name);
-			mpt::String::Copy(sample.filename, name);
+			sample.filename = name;
 		}
 
 		MO3Sample smpHeader;
@@ -1321,11 +1334,11 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 		{
 			// Uncompressed sample
 			SampleIO(
-				(smpHeader.flags & MO3Sample::smp16Bit) ? SampleIO::_16bit : SampleIO::_8bit,
-				(smpHeader.flags & MO3Sample::smpStereo) ? SampleIO::stereoSplit : SampleIO::mono,
-				SampleIO::littleEndian,
-				SampleIO::signedPCM)
-				.ReadSample(Samples[smp], file);
+			    (smpHeader.flags & MO3Sample::smp16Bit) ? SampleIO::_16bit : SampleIO::_8bit,
+			    (smpHeader.flags & MO3Sample::smpStereo) ? SampleIO::stereoSplit : SampleIO::mono,
+			    SampleIO::littleEndian,
+			    SampleIO::signedPCM)
+			    .ReadSample(Samples[smp], file);
 		} else if(smpHeader.compressedSize < 0 && (smp + smpHeader.compressedSize) > 0)
 		{
 			// Duplicate sample
@@ -1339,8 +1352,10 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 			}
 		} else if(smpHeader.compressedSize > 0)
 		{
-			if(smpHeader.flags & MO3Sample::smp16Bit) sample.uFlags.set(CHN_16BIT);
-			if(smpHeader.flags & MO3Sample::smpStereo) sample.uFlags.set(CHN_STEREO);
+			if(smpHeader.flags & MO3Sample::smp16Bit)
+				sample.uFlags.set(CHN_16BIT);
+			if(smpHeader.flags & MO3Sample::smpStereo)
+				sample.uFlags.set(CHN_STEREO);
 
 			FileReader sampleData = file.ReadChunk(smpHeader.compressedSize);
 			const uint8 numChannels = sample.GetNumChannels();
@@ -1349,7 +1364,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				// In the best case, MO3 compression represents each sample point as two bits.
 				// As a result, if we have a file length of n, we know that the sample can be at most n*4 sample points long.
-				size_t maxLength = smpHeader.compressedSize;
+				auto maxLength = sampleData.GetLength();
 				uint8 maxSamplesPerByte = 4 / numChannels;
 				if(Util::MaxValueOfType(maxLength) / maxSamplesPerByte >= maxLength)
 					maxLength *= maxSamplesPerByte;
@@ -1465,7 +1480,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				// We could in theory only adjust the header and pass 2 chunks to libvorbisfile.
 				// Another option would be to demux both chunks on our own (or using libogg) and pass the raw packet data to libvorbis directly.
 
-				mpt::ostringstream mergedStream(std::ios::binary);
+				std::ostringstream mergedStream(std::ios::binary);
 				mergedStream.imbue(std::locale::classic());
 
 				sampleChunks[sharedOggHeader - 1].chunk.Rewind();
@@ -1514,7 +1529,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				// We assume same ordering of streams in both header and data if
 				// multiple streams are present.
 
-				mpt::ostringstream mergedStream(std::ios::binary);
+				std::ostringstream mergedStream(std::ios::binary);
 				mergedStream.imbue(std::locale::classic());
 
 				sampleChunks[sharedOggHeader - 1].chunk.Rewind();
@@ -1590,7 +1605,8 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 
 				sampleChunk.chunk.Rewind();
 				FileReader::PinnedRawDataView sampleChunkView = sampleChunk.chunk.GetPinnedRawDataView();
-				mergedData.insert(mergedData.end(), mpt::byte_cast<const char *>(sampleChunkView.begin()), mpt::byte_cast<const char *>(sampleChunkView.end()));
+				mpt::span<const char> sampleChunkViewSpan = mpt::byte_cast<mpt::span<const char>>(sampleChunkView.span());
+				mergedData.insert(mergedData.end(), sampleChunkViewSpan.begin(), sampleChunkViewSpan.end());
 
 #endif
 			}
@@ -1599,15 +1615,15 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 			FileReader &sampleData = sharedHeader ? mergedDataChunk : sampleChunk.chunk;
 			FileReader &headerChunk = sampleData;
 
-#else // !(MPT_WITH_VORBIS && MPT_WITH_VORBISFILE)
+#else  // !(MPT_WITH_VORBIS && MPT_WITH_VORBISFILE)
 
 			FileReader &sampleData = sampleChunk.chunk;
 			FileReader &headerChunk = sharedHeader ? sampleChunks[sharedOggHeader - 1].chunk : sampleData;
 #if defined(MPT_WITH_STBVORBIS)
 			std::size_t initialRead = sharedHeader ? sampleChunk.headerSize : headerChunk.GetLength();
-#endif // MPT_WITH_STBVORBIS
+#endif  // MPT_WITH_STBVORBIS
 
-#endif // MPT_WITH_VORBIS && MPT_WITH_VORBISFILE
+#endif  // MPT_WITH_VORBIS && MPT_WITH_VORBISFILE
 
 			headerChunk.Rewind();
 			if(sharedHeader && !headerChunk.CanRead(sampleChunk.headerSize))
@@ -1616,17 +1632,16 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 #if defined(MPT_WITH_VORBIS) && defined(MPT_WITH_VORBISFILE)
 
 			ov_callbacks callbacks = {
-				&VorbisfileFilereaderRead,
-				&VorbisfileFilereaderSeek,
-				NULL,
-				&VorbisfileFilereaderTell
-			};
+			    &VorbisfileFilereaderRead,
+			    &VorbisfileFilereaderSeek,
+			    nullptr,
+			    &VorbisfileFilereaderTell};
 			OggVorbis_File vf;
 			MemsetZero(vf);
 			if(ov_open_callbacks(&sampleData, &vf, nullptr, 0, callbacks) == 0)
 			{
 				if(ov_streams(&vf) == 1)
-				{ // we do not support chained vorbis samples
+				{  // we do not support chained vorbis samples
 					vorbis_info *vi = ov_info(&vf, -1);
 					if(vi && vi->rate > 0 && vi->channels > 0)
 					{
@@ -1701,7 +1716,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				headerChunk.Skip(consumed);
 			}
 			FileReader::PinnedRawDataView sampleDataView = sampleData.GetPinnedRawDataView();
-			const mpt::byte* data = sampleDataView.data();
+			const std::byte *data = sampleDataView.data();
 			std::size_t dataLeft = sampleDataView.size();
 			if(!sharedHeader)
 			{
@@ -1717,7 +1732,7 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				sample.AllocateSample();
 				SmpLength offset = 0;
 				while((error == VORBIS__no_error || (error == VORBIS_need_more_data && dataLeft > 0))
-					&& offset < sample.nLength && sample.HasSampleData())
+				      && offset < sample.nLength && sample.HasSampleData())
 				{
 					int channels = 0, decodedSamples = 0;
 					float **output;
@@ -1745,11 +1760,11 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				unsupportedSamples = true;
 			}
 
-#else // !VORBIS
+#else  // !VORBIS
 
 			unsupportedSamples = true;
 
-#endif // VORBIS
+#endif  // VORBIS
 		}
 	}
 
@@ -1785,14 +1800,13 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				ReadMixPluginChunk(pluginChunk, m_MixPlugins[plug - 1]);
 			}
-#endif // NO_PLUGINS
+#endif  // NO_PLUGINS
 		}
 	}
 
 	mpt::ustring madeWithTracker;
 	uint16 cwtv = 0;
 	uint16 cmwt = 0;
-	MPT_UNUSED_VARIABLE(cmwt);
 	while(musicChunk.CanRead(8))
 	{
 		uint32 id = musicChunk.ReadUint32LE();
@@ -1816,14 +1830,14 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				cwtv = chunk.ReadUint16LE();
 				break;
 			case MOD_TYPE_XM:
-				chunk.ReadString<mpt::String::spacePadded>(madeWithTracker, mpt::CharsetCP437, std::min(FileReader::off_t(32), chunk.GetLength()));
+				chunk.ReadString<mpt::String::spacePadded>(madeWithTracker, mpt::Charset::CP437, std::min(FileReader::off_t(32), chunk.GetLength()));
 				break;
 			case MOD_TYPE_MTM:
-				{
-					uint8 mtmVersion = chunk.ReadUint8();
-					madeWithTracker = mpt::format(U_("MultiTracker %1.%2"))(mtmVersion >> 4, mtmVersion & 0x0F);
-				}
-				break;
+			{
+				uint8 mtmVersion = chunk.ReadUint8();
+				madeWithTracker = mpt::format(U_("MultiTracker %1.%2"))(mtmVersion >> 4, mtmVersion & 0x0F);
+			}
+			break;
 			default:
 				break;
 			}
@@ -1880,8 +1894,8 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	if((GetType() == MOD_TYPE_IT && cwtv >= 0x0100 && cwtv < 0x0214)
-		|| (GetType() == MOD_TYPE_S3M && cwtv >= 0x3100 && cwtv < 0x3214)
-		|| (GetType() == MOD_TYPE_S3M && cwtv >= 0x1300 && cwtv < 0x1320))
+	   || (GetType() == MOD_TYPE_S3M && cwtv >= 0x3100 && cwtv < 0x3214)
+	   || (GetType() == MOD_TYPE_S3M && cwtv >= 0x1300 && cwtv < 0x1320))
 	{
 		// Ignore MIDI data in files made with IT older than version 2.14 and old ST3 versions.
 		m_MidiCfg.ClearZxxMacros();
@@ -1904,14 +1918,14 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				}
 			}
 		}
-		if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 18, 00, 00))
+		if(m_dwLastSavedWithVersion < MPT_V("1.18.00.00"))
 		{
 			m_playBehaviour.reset(kITOffset);
-			m_playBehaviour.reset(kFT2OffsetOutOfRange);
+			m_playBehaviour.reset(kFT2ST3OffsetOutOfRange);
 		}
-		if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 23, 00, 00))
+		if(m_dwLastSavedWithVersion < MPT_V("1.23.00.00"))
 			m_playBehaviour.reset(kFT2Periods);
-		if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 26, 00, 00))
+		if(m_dwLastSavedWithVersion < MPT_V("1.26.00.00"))
 			m_playBehaviour.reset(kITInstrWithNoteOff);
 	}
 
@@ -1922,15 +1936,46 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 
 	m_modFormat.formatName = mpt::format(U_("Un4seen MO3 v%1"))(version);
 	m_modFormat.type = U_("mo3");
-	m_modFormat.originalType = std::move(originalFormatType);
-	m_modFormat.originalFormatName = std::move(originalFormatName);
+
+	switch(GetType())
+	{
+	case MOD_TYPE_MTM:
+		m_modFormat.originalType = U_("mtm");
+		m_modFormat.originalFormatName = U_("MultiTracker");
+		break;
+	case MOD_TYPE_MOD:
+		m_modFormat.originalType = U_("mod");
+		m_modFormat.originalFormatName = U_("Generic MOD");
+		break;
+	case MOD_TYPE_XM:
+		m_modFormat.originalType = U_("xm");
+		m_modFormat.originalFormatName = U_("FastTracker 2");
+		break;
+	case MOD_TYPE_S3M:
+		m_modFormat.originalType = U_("s3m");
+		m_modFormat.originalFormatName = U_("ScreamTracker 3");
+		break;
+	case MOD_TYPE_IT:
+		m_modFormat.originalType = U_("it");
+		if(cmwt)
+			m_modFormat.originalFormatName = mpt::format(U_("Impulse Tracker %1.%2"))(cmwt >> 8, mpt::ufmt::hex0<2>(cmwt & 0xFF));
+		else
+			m_modFormat.originalFormatName = U_("Impulse Tracker");
+		break;
+	case MOD_TYPE_MPT:
+		m_modFormat.originalType = U_("mptm");
+		m_modFormat.originalFormatName = U_("OpenMPT MPTM");
+		break;
+	default:
+		MPT_ASSERT_NOTREACHED();
+	}
 	m_modFormat.madeWithTracker = std::move(madeWithTracker);
 	if(m_dwLastSavedWithVersion)
-		m_modFormat.charset = mpt::CharsetWindows1252;
+		m_modFormat.charset = mpt::Charset::Windows1252;
 	else if(GetType() == MOD_TYPE_MOD)
-		m_modFormat.charset = mpt::CharsetISO8859_1;
+		m_modFormat.charset = mpt::Charset::ISO8859_1;
 	else
-		m_modFormat.charset = mpt::CharsetCP437;
+		m_modFormat.charset = mpt::Charset::CP437;
 
 	if(unsupportedSamples)
 	{
