@@ -19,7 +19,7 @@
 #ifdef MPT_EXTERNAL_SAMPLES
 // For loading external data in Startrekker files
 #include "../common/mptPathString.h"
-#endif // MPT_EXTERNAL_SAMPLES
+#endif  // MPT_EXTERNAL_SAMPLES
 
 OPENMPT_NAMESPACE_BEGIN
 
@@ -61,6 +61,7 @@ void CSoundFile::ConvertModCommand(ModCommand &m)
 	case 'P' - 55:	m.command = CMD_PANNINGSLIDE; break;
 	case 'R' - 55:	m.command = CMD_RETRIG; break;
 	case 'T' - 55:	m.command = CMD_TREMOR; break;
+	case 'W' - 55:	m.command = CMD_DUMMY; break;
 	case 'X' - 55:	m.command = CMD_XFINEPORTAUPDOWN;	break;
 	case 'Y' - 55:	m.command = CMD_PANBRELLO; break;			//34
 	case 'Z' - 55:	m.command = CMD_MIDI;	break;				//35
@@ -106,7 +107,7 @@ void CSoundFile::ModSaveCommand(uint8 &command, uint8 &param, bool toXM, bool co
 		{
 			if(param <= 0x80)
 			{
-				param = MIN(param << 1, 0xFF);
+				param = mpt::saturate_cast<uint8>(param * 2);
 			}
 			else if(param == 0xA4)	// surround
 			{
@@ -128,8 +129,8 @@ void CSoundFile::ModSaveCommand(uint8 &command, uint8 &param, bool toXM, bool co
 	case CMD_VOLUME:			command = 0x0C; break;
 	case CMD_PATTERNBREAK:		command = 0x0D; param = ((param / 10) << 4) | (param % 10); break;
 	case CMD_MODCMDEX:			command = 0x0E; break;
-	case CMD_SPEED:				command = 0x0F; param = std::min<uint8>(param, 0x1F); break;
-	case CMD_TEMPO:				command = 0x0F; param = std::max<uint8>(param, 0x20); break;
+	case CMD_SPEED:				command = 0x0F; param = std::min(param, uint8(0x1F)); break;
+	case CMD_TEMPO:				command = 0x0F; param = std::max(param, uint8(0x20)); break;
 	case CMD_GLOBALVOLUME:		command = 'G' - 55; break;
 	case CMD_GLOBALVOLSLIDE:	command = 'H' - 55; break;
 	case CMD_KEYOFF:			command = 'K' - 55; break;
@@ -137,6 +138,7 @@ void CSoundFile::ModSaveCommand(uint8 &command, uint8 &param, bool toXM, bool co
 	case CMD_PANNINGSLIDE:		command = 'P' - 55; break;
 	case CMD_RETRIG:			command = 'R' - 55; break;
 	case CMD_TREMOR:			command = 'T' - 55; break;
+	case CMD_DUMMY:				command = 'W' - 55; break;
 	case CMD_XFINEPORTAUPDOWN:	command = 'X' - 55;
 		if(compatibilityExport && param >= 0x30)	// X1x and X2x are legit, everything above are MPT extensions, which don't belong here.
 			param = 0;	// Don't set command to 0 to indicate that there *was* some X command here...
@@ -197,7 +199,7 @@ void CSoundFile::ModSaveCommand(uint8 &command, uint8 &param, bool toXM, bool co
 	}
 }
 
-#endif // MODPLUG_NO_FILESAVE
+#endif  // MODPLUG_NO_FILESAVE
 
 
 // File Header
@@ -227,13 +229,13 @@ struct MODSampleHeader
 		mptSmp.Initialize(MOD_TYPE_MOD);
 		mptSmp.nLength = length * 2;
 		mptSmp.nFineTune = MOD2XMFineTune(finetune & 0x0F);
-		mptSmp.nVolume = 4u * std::min<uint8>(volume, 64);
+		mptSmp.nVolume = 4u * std::min(volume.get(), uint8(64));
 
 		SmpLength lStart = loopStart * 2;
 		SmpLength lLength = loopLength * 2;
 		// See if loop start is incorrect as words, but correct as bytes (like in Soundtracker modules)
 		if(lLength > 2 && (lStart + lLength > mptSmp.nLength)
-			&& (lStart / 2 + lLength <= mptSmp.nLength))
+		   && (lStart / 2 + lLength <= mptSmp.nLength))
 		{
 			lStart /= 2;
 		}
@@ -315,19 +317,19 @@ struct MODSampleHeader
 	uint32 GetInvalidByteScore() const
 	{
 		return ((volume > 64) ? 1 : 0)
-			+ ((finetune > 15) ? 1 : 0)
-			+ ((loopStart > length * 2) ? 1 : 0);
+		       + ((finetune > 15) ? 1 : 0)
+		       + ((loopStart > length * 2) ? 1 : 0);
 	}
 
 	// Suggested threshold for rejecting invalid files based on cumulated score returned by GetInvalidByteScore
-	enum : uint32 { INVALID_BYTE_THRESHOLD = 40 };
-	
+	static constexpr uint32 INVALID_BYTE_THRESHOLD = 40;
+
 	// This threshold is used for files where the file magic only gives a
 	// fragile result which alone would lead to too many false positives.
 	// In particular, the files from Inconexia demo by Iguana
 	// (https://www.pouet.net/prod.php?which=830) which have 3 \0 bytes in
 	// the file magic tend to cause misdetection of random files.
-	enum : uint32 { INVALID_BYTE_FRAGILE_THRESHOLD = 1 };
+	static constexpr uint32 INVALID_BYTE_FRAGILE_THRESHOLD = 1;
 
 	// Retrieve the internal sample format flags for this sample.
 	static SampleIO GetSampleFormat()
@@ -342,6 +344,9 @@ struct MODSampleHeader
 
 MPT_BINARY_STRUCT(MODSampleHeader, 30)
 
+// Pattern data of a 4-channel MOD file
+using MODPatternData = std::array<std::array<std::array<uint8, 4>, 4>, 64>;
+
 // Synthesized StarTrekker instruments
 struct AMInstrument
 {
@@ -369,7 +374,7 @@ struct AMInstrument
 		sample.nLoopStart = 0;
 		sample.nLoopEnd = sample.nLength;
 		sample.uFlags.set(CHN_LOOP);
-		sample.nVolume = 256;	// prelude.mod has volume 0 in sample header
+		sample.nVolume = 256;  // prelude.mod has volume 0 in sample header
 		sample.nVibDepth = mpt::saturate_cast<uint8>(vibAmp * 2);
 		sample.nVibRate = static_cast<uint8>(vibSpeed);
 		sample.nVibType = VIB_SINE;
@@ -398,7 +403,7 @@ struct AMInstrument
 		const struct
 		{
 			uint16 level, speed;
-		} points[] = { { startLevel, 0 }, { attack1Level, attack1Speed }, { attack2Level, attack2Speed }, { sustainLevel, decaySpeed }, { sustainLevel, sustainTime }, { 0, releaseSpeed } };
+		} points[] = {{startLevel, 0}, {attack1Level, attack1Speed}, {attack2Level, attack2Speed}, {sustainLevel, decaySpeed}, {sustainLevel, sustainTime}, {0, releaseSpeed}};
 
 		for(uint8 i = 1; i < CountOf(points); i++)
 		{
@@ -499,14 +504,14 @@ static bool IsMagic(const char *magic1, const char (&magic2)[5])
 }
 
 
-static uint32 ReadSample(FileReader &file, MODSampleHeader &sampleHeader, ModSample &sample, char (&sampleName)[MAX_SAMPLENAME], bool is4Chn)
+static uint32 ReadSample(FileReader &file, MODSampleHeader &sampleHeader, ModSample &sample, mpt::charbuf<MAX_SAMPLENAME> &sampleName, bool is4Chn)
 {
 	file.ReadStruct(sampleHeader);
 	sampleHeader.ConvertToMPT(sample, is4Chn);
 
-	mpt::String::Read<mpt::String::spacePadded>(sampleName, sampleHeader.name);
+	sampleName = mpt::String::ReadBuf(mpt::String::spacePadded, sampleHeader.name);
 	// Get rid of weird characters in sample names.
-	for(auto &c : sampleName)
+	for(auto &c : sampleName.buf)
 	{
 		if(c > 0 && c < ' ')
 		{
@@ -518,8 +523,36 @@ static uint32 ReadSample(FileReader &file, MODSampleHeader &sampleHeader, ModSam
 }
 
 
+// Count malformed bytes in MOD pattern data
+static uint32 CountMalformedMODPatternData(const MODPatternData &patternData, const bool allow31Samples)
+{
+	const uint8 mask = allow31Samples ? 0xE0 : 0xF0;
+	uint32 malformedBytes = 0;
+	for(const auto &row : patternData)
+	{
+		for(const auto &data : row)
+		{
+			if(data[0] & mask)
+				malformedBytes++;
+		}
+	}
+	return malformedBytes;
+}
+
+
+// Check if number of malformed bytes in MOD pattern data exceeds some threshold
+template <typename TFileReader>
+static bool ValidateMODPatternData(TFileReader &file, const uint32 threshold, const bool allow31Samples)
+{
+	MODPatternData patternData;
+	if(!file.Read(patternData))
+		return false;
+	return CountMalformedMODPatternData(patternData, allow31Samples) <= threshold;
+}
+
+
 // Parse the order list to determine how many patterns are used in the file.
-static PATTERNINDEX GetNumPatterns(FileReader &file, ModSequence &Order, ORDERINDEX numOrders, SmpLength totalSampleLen, CHANNELINDEX &numChannels, bool checkForWOW)
+static PATTERNINDEX GetNumPatterns(FileReader &file, ModSequence &Order, ORDERINDEX numOrders, SmpLength totalSampleLen, CHANNELINDEX &numChannels, SmpLength wowSampleLen, bool validateHiddenPatterns)
 {
 	PATTERNINDEX numPatterns = 0;         // Total number of patterns in file (determined by going through the whole order list) with pattern number < 128
 	PATTERNINDEX officialPatterns = 0;    // Number of patterns only found in the "official" part of the order list (i.e. order positions < claimed order length)
@@ -548,12 +581,16 @@ static PATTERNINDEX GetNumPatterns(FileReader &file, ModSequence &Order, ORDERIN
 	const size_t patternStartOffset = file.GetPosition();
 	const size_t sizeWithoutPatterns = totalSampleLen + patternStartOffset;
 
-	if(checkForWOW && sizeWithoutPatterns + numPatterns * 8 * 256 == file.GetLength())
+	if(wowSampleLen && (wowSampleLen + patternStartOffset) + numPatterns * 8 * 256 == (file.GetLength() & ~1))
 	{
-		// Check if this is a Mod's Grave WOW file... Never seen one of those, but apparently they *do* exist.
-		// WOW files should use the M.K. magic but are actually 8CHN files.
-		numChannels = 8;
-	} else if(numPatterns != officialPatterns && numChannels == 4 && !checkForWOW)
+		// Check if this is a Mod's Grave WOW file... WOW files use the M.K. magic but are actually 8CHN files.
+		// We do a simple pattern validation as well for regular MOD files that have non-module data attached at the end
+		// (e.g. ponylips.mod, MD5 c039af363b1d99a492dafc5b5f9dd949, SHA1 1bee1941c47bc6f913735ce0cf1880b248b8fc93)
+		file.Seek(patternStartOffset + numPatterns * 4 * 256);
+		if(ValidateMODPatternData(file, 16, true))
+			numChannels = 8;
+		file.Seek(patternStartOffset);
+	} else if(numPatterns != officialPatterns && validateHiddenPatterns)
 	{
 		// Fix SoundTracker modules where "hidden" patterns should be ignored.
 		// razor-1911.mod (MD5 b75f0f471b0ae400185585ca05bf7fe8, SHA1 4de31af234229faec00f1e85e1e8f78f405d454b)
@@ -568,21 +605,8 @@ static PATTERNINDEX GetNumPatterns(FileReader &file, ModSequence &Order, ORDERIN
 		// Hence, we have a peek at the first hidden pattern and check if it contains a lot of illegal data.
 		// If that is the case, we assume it's part of the sample data and only consider the "official" patterns.
 		file.Seek(patternStartOffset + officialPatterns * 1024);
-		int illegalBytes = 0;
-		for(int i = 0; i < 256; i++)
-		{
-			uint8 data[4];
-			file.ReadArray(data);
-			if(data[0] & 0xE0)
-			{
-				illegalBytes++;
-				if(illegalBytes > 64)
-				{
-					numPatterns = officialPatterns;
-					break;
-				}
-			}
-		}
+		if(!ValidateMODPatternData(file, 64, true))
+			numPatterns = officialPatterns;
 		file.Seek(patternStartOffset);
 	}
 
@@ -620,21 +644,19 @@ static PATTERNINDEX GetNumPatterns(FileReader &file, ModSequence &Order, ORDERIN
 
 void CSoundFile::ReadMODPatternEntry(FileReader &file, ModCommand &m)
 {
-	uint8 data[4];
-	file.ReadArray(data);
-	ReadMODPatternEntry(data, m);
+	ReadMODPatternEntry(file.ReadArray<uint8, 4>(), m);
 }
 
 
-void CSoundFile::ReadMODPatternEntry(const uint8 (&data)[4], ModCommand &m)
+void CSoundFile::ReadMODPatternEntry(const std::array<uint8, 4> data, ModCommand &m)
 {
 	// Read Period
 	uint16 period = (((static_cast<uint16>(data[0]) & 0x0F) << 8) | data[1]);
 	size_t note = NOTE_NONE;
 	if(period > 0 && period != 0xFFF)
 	{
-		note = mpt::size(ProTrackerPeriodTable) + 23 + NOTE_MIN;
-		for(size_t i = 0; i < mpt::size(ProTrackerPeriodTable); i++)
+		note = std::size(ProTrackerPeriodTable) + 23 + NOTE_MIN;
+		for(size_t i = 0; i < std::size(ProTrackerPeriodTable); i++)
 		{
 			if(period >= ProTrackerPeriodTable[i])
 			{
@@ -664,7 +686,7 @@ void CSoundFile::ReadMODPatternEntry(const uint8 (&data)[4], ModCommand &m)
 
 struct MODMagicResult
 {
-	const MPT_UCHAR_TYPE *madeWithTracker = nullptr;
+	const mpt::uchar *madeWithTracker = nullptr;
 	uint32 invalidByteThreshold = MODSampleHeader::INVALID_BYTE_THRESHOLD;
 	CHANNELINDEX numChannels    = 0;
 	bool isNoiseTracker         = false;
@@ -676,29 +698,29 @@ struct MODMagicResult
 
 static bool CheckMODMagic(const char magic[4], MODMagicResult &result)
 {
-	if(IsMagic(magic, "M.K.")		// ProTracker and compatible
-		|| IsMagic(magic, "M!K!")	// ProTracker (>64 patterns)
-		|| IsMagic(magic, "PATT")	// ProTracker 3.6
-		|| IsMagic(magic, "NSMS")	// kingdomofpleasure.mod by bee hunter
-		|| IsMagic(magic, "LARD"))	// judgement_day_gvine.mod by 4-mat
+	if(IsMagic(magic, "M.K.")      // ProTracker and compatible
+	   || IsMagic(magic, "M!K!")   // ProTracker (>64 patterns)
+	   || IsMagic(magic, "PATT")   // ProTracker 3.6
+	   || IsMagic(magic, "NSMS")   // kingdomofpleasure.mod by bee hunter
+	   || IsMagic(magic, "LARD"))  // judgement_day_gvine.mod by 4-mat
 	{
 		result.madeWithTracker = UL_("Generic ProTracker or compatible");
 		result.numChannels = 4;
-	} else if(IsMagic(magic, "M&K!")	// "His Master's Noise" musicdisk
-		|| IsMagic(magic, "FEST")		// "His Master's Noise" musicdisk
-		|| IsMagic(magic, "N.T."))
+	} else if(IsMagic(magic, "M&K!")     // "His Master's Noise" musicdisk
+	          || IsMagic(magic, "FEST")  // "His Master's Noise" musicdisk
+	          || IsMagic(magic, "N.T."))
 	{
 		result.madeWithTracker = UL_("NoiseTracker");
 		result.isNoiseTracker = true;
 		result.numChannels = 4;
 	} else if(IsMagic(magic, "OKTA")
-		|| IsMagic(magic, "OCTA"))
+	          || IsMagic(magic, "OCTA"))
 	{
 		// Oktalyzer
 		result.madeWithTracker = UL_("Oktalyzer");
 		result.numChannels = 8;
 	} else if(IsMagic(magic, "CD81")
-		|| IsMagic(magic, "CD61"))
+	          || IsMagic(magic, "CD61"))
 	{
 		// Octalyser on Atari STe/Falcon
 		result.madeWithTracker = UL_("Octalyser (Atari)");
@@ -727,8 +749,8 @@ static bool CheckMODMagic(const char magic[4], MODMagicResult &result)
 		result.madeWithTracker = UL_("Generic MOD-compatible Tracker");
 		result.isGenericMultiChannel = true;
 		result.numChannels = magic[0] - '0';
-	} else if(magic[0] >= '1' && magic[0] <= '9' && magic[1]>='0' && magic[1] <= '9'
-		&& (!memcmp(magic + 2, "CH", 2) || !memcmp(magic + 2, "CN", 2)))
+	} else if(magic[0] >= '1' && magic[0] <= '9' && magic[1] >= '0' && magic[1] <= '9'
+	          && (!memcmp(magic + 2, "CH", 2) || !memcmp(magic + 2, "CN", 2)))
 	{
 		// xxCN / xxCH - Many trackers
 		result.madeWithTracker = UL_("Generic MOD-compatible Tracker");
@@ -792,8 +814,8 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 
 	MODMagicResult modMagicResult;
 	if(!CheckMODMagic(magic, modMagicResult)
-		|| modMagicResult.numChannels < 1
-		|| modMagicResult.numChannels > MAX_BASECHANNELS)
+	   || modMagicResult.numChannels < 1
+	   || modMagicResult.numChannels > MAX_BASECHANNELS)
 	{
 		return false;
 	}
@@ -824,13 +846,14 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	const bool isMdKd = IsMagic(magic, "M.K.");
 	// Adjust finetune values for modules saved with "His Master's Noisetracker"
 	const bool isHMNT = IsMagic(magic, "M&K!") || IsMagic(magic, "FEST");
+	bool maybeWOW = isMdKd;
 
 	// Reading song title
 	file.Seek(0);
 	file.ReadString<mpt::String::spacePadded>(m_songName, 20);
 
 	// Load Sample Headers
-	SmpLength totalSampleLen = 0;
+	SmpLength totalSampleLen = 0, wowSampleLen = 0;
 	m_nSamples = 31;
 	uint32 invalidBytes = 0;
 	for(SAMPLEINDEX smp = 1; smp <= 31; smp++)
@@ -840,16 +863,25 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 		totalSampleLen += Samples[smp].nLength;
 
 		if(isHMNT)
-		{
 			Samples[smp].nFineTune = -static_cast<int8>(sampleHeader.finetune << 3);
-		} else if(Samples[smp].nLength > 65535)
-		{
+		else if(Samples[smp].nLength > 65535)
 			isNoiseTracker = false;
-		}
+		
 		if(sampleHeader.length && !sampleHeader.loopLength)
 			hasRepLen0 = true;
 		else if(!sampleHeader.length && sampleHeader.volume == 64)
 			hasEmptySampleWithVolume = true;
+
+		if(maybeWOW)
+		{
+			// Some WOW files rely on sample length 1 being counted as well
+			wowSampleLen += sampleHeader.length * 2;
+			// WOW files are converted 669 files, which don't support finetune or default volume
+			if(sampleHeader.finetune)
+				maybeWOW = false;
+			else if(sampleHeader.length > 0 && sampleHeader.volume != 64)
+				maybeWOW = false;
+		}
 	}
 	// If there is too much binary garbage in the sample headers, reject the file.
 	if(invalidBytes > modMagicResult.invalidByteThreshold)
@@ -860,7 +892,12 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	// Read order information
 	MODFileHeader fileHeader;
 	file.ReadStruct(fileHeader);
-	file.Skip(4);	// Magic bytes (we already parsed these)
+	file.Skip(4);  // Magic bytes (we already parsed these)
+
+	if(fileHeader.restartPos > 0)
+		maybeWOW = false;
+	if(!maybeWOW)
+		wowSampleLen = 0;
 
 	ReadOrderFromArray(Order(), fileHeader.orderList);
 
@@ -880,11 +917,12 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	// Get number of patterns (including some order list sanity checks)
-	PATTERNINDEX numPatterns = GetNumPatterns(file, Order(), realOrders, totalSampleLen, m_nChannels, isMdKd);
-	if(isMdKd && GetNumChannels() == 8)
+	PATTERNINDEX numPatterns = GetNumPatterns(file, Order(), realOrders, totalSampleLen, m_nChannels, wowSampleLen, false);
+	if(maybeWOW && GetNumChannels() == 8)
 	{
-		// M.K. with 8 channels = Grave Composer
+		// M.K. with 8 channels = Mod's Grave
 		modMagicResult.madeWithTracker = UL_("Mod's Grave");
+		isGenericMultiChannel = true;
 	}
 
 	if(isFLT8)
@@ -929,10 +967,11 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	// - Scan patterns to check if file could be a NoiseTracker file in disguise.
 	//   In this case, the parameter of Dxx commands needs to be ignored.
 	// - Use the same code to find notes that would be out-of-range on Amiga.
-	// - Detect 7-bit panning.
+	// - Detect 7-bit panning and whether 8xx / E8x commands should be interpreted as panning at all.
 	bool onlyAmigaNotes = true;
 	bool fix7BitPanning = false;
 	uint8 maxPanning = 0;  // For detecting 8xx-as-sync
+	const uint8 ENABLE_MOD_PANNING_THRESHOLD = 0x30;
 	if(!isNoiseTracker)
 	{
 		bool leftPanning = false, extendedPanning = false;  // For detecting 800-880 panning
@@ -965,11 +1004,11 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 						extendedPanning = true;
 				} else if(m.command == 0x0E && (m.param & 0xF0) == 0x80)
 				{
-					maxPanning = std::max<uint8>(maxPanning, (m.param & 0x0F) << 4);
+					maxPanning = std::max(maxPanning, static_cast<uint8>((m.param & 0x0F) << 4));
 				}
 			}
 		}
-		fix7BitPanning = leftPanning && !extendedPanning;
+		fix7BitPanning = leftPanning && !extendedPanning && maxPanning >= ENABLE_MOD_PANNING_THRESHOLD;
 	}
 	file.Seek(1084);
 
@@ -1091,7 +1130,8 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 					lastInstrument[chn] = m.instr;
 				}
 			}
-			if(hasSpeedOnRow && hasTempoOnRow) definitelyCIA = true;
+			if(hasSpeedOnRow && hasTempoOnRow)
+				definitelyCIA = true;
 		}
 	}
 
@@ -1105,7 +1145,7 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 		m_playBehaviour.set(kMODOutOfRangeNoteDelay);
 		m_playBehaviour.set(kMODTempoOnSecondTick);
 		// Arbitrary threshold for deciding that 8xx effects are only used as sync markers
-		if(maxPanning < 0x20)
+		if(maxPanning < ENABLE_MOD_PANNING_THRESHOLD)
 		{
 			m_playBehaviour.set(kMODIgnorePanning);
 			if(fileHeader.restartPos != 0x7F)
@@ -1124,6 +1164,10 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	{
 		m_SongFlags.set(SONG_ISAMIGA);
 	}
+	if(isGenericMultiChannel || isMdKd)
+	{
+		m_playBehaviour.set(kFT2MODTremoloRampWaveform);
+	}
 	if(isInconexia)
 	{
 		m_playBehaviour.set(kMODIgnorePanning);
@@ -1179,12 +1223,12 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 		if(!filename.empty())
 		{
 			// Find instrument definition file
-			const mpt::PathString exts[] = { P_(".nt"), P_(".NT"), P_(".as"), P_(".AS") };
+			const mpt::PathString exts[] = {P_(".nt"), P_(".NT"), P_(".as"), P_(".AS")};
 			for(const auto &ext : exts)
 			{
 				mpt::PathString infoName = filename + ext;
 				char stMagic[16];
-				if(infoName.IsFile() && amFile.Open(infoName) && (amData = GetFileReader(amFile)).IsValid() && amData.ReadArray(stMagic))
+				if(infoName.IsFile() && amFile.Open(infoName, SettingCacheCompleteFileBeforeLoading()) && (amData = GetFileReader(amFile)).IsValid() && amData.ReadArray(stMagic))
 				{
 					if(!memcmp(stMagic, "ST1.2 ModuleINFO", 16))
 						modMagicResult.madeWithTracker = UL_("Startrekker 1.2");
@@ -1218,7 +1262,7 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				break;
 			}
-			mpt::String::Copy(ins->name, m_szNames[smp]);
+			ins->name = m_szNames[smp];
 
 			AMInstrument am;
 			// Allow partial reads for fa.worse face.mod
@@ -1231,7 +1275,7 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 			amData.Skip(120 - sizeof(AMInstrument));
 		}
 	}
-#endif // MPT_EXTERNAL_SAMPLES || MPT_BUILD_FUZZER
+#endif  // MPT_EXTERNAL_SAMPLES || MPT_BUILD_FUZZER
 
 	// Fix VBlank MODs. Arbitrary threshold: 10 minutes.
 	// Basically, this just converts all tempo commands into speed commands
@@ -1262,17 +1306,18 @@ bool CSoundFile::ReadMOD(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	std::transform(std::begin(magic), std::end(magic), std::begin(magic), [](unsigned char c) -> unsigned char { return (c < ' ') ? ' ' : c; });
-	m_modFormat.formatName = mpt::format(U_("ProTracker MOD (%1)"))(mpt::ToUnicode(mpt::CharsetASCII, std::string(std::begin(magic), std::end(magic))));
+	m_modFormat.formatName = mpt::format(U_("ProTracker MOD (%1)"))(mpt::ToUnicode(mpt::Charset::ASCII, std::string(std::begin(magic), std::end(magic))));
 	m_modFormat.type = U_("mod");
-	if(modMagicResult.madeWithTracker) m_modFormat.madeWithTracker = modMagicResult.madeWithTracker;
-	m_modFormat.charset = mpt::CharsetISO8859_1;
+	if(modMagicResult.madeWithTracker)
+		m_modFormat.madeWithTracker = modMagicResult.madeWithTracker;
+	m_modFormat.charset = mpt::Charset::ISO8859_1;
 
 	return true;
 }
 
 
 // Check if a name string is valid (i.e. doesn't contain binary garbage data)
-template<size_t N>
+template <size_t N>
 static uint32 CountInvalidChars(const char (&name)[N])
 {
 	uint32 invalidChars = 0;
@@ -1291,13 +1336,13 @@ static uint32 CountInvalidChars(const char (&name)[N])
 // Thanks for Fraggie for this information! (https://www.un4seen.com/forum/?topic=14471.msg100829#msg100829)
 enum STVersions
 {
-	UST1_00,             // Ultimate Soundtracker 1.0-1.21 (K. Obarski)
-	UST1_80,             // Ultimate Soundtracker 1.8-2.0 (K. Obarski)
-	ST2_00_Exterminator, // SoundTracker 2.0 (The Exterminator), D.O.C. Sountracker II (Unknown/D.O.C.)
-	ST_III,              // Defjam Soundtracker III (Il Scuro/Defjam), Alpha Flight SoundTracker IV (Alpha Flight), D.O.C. SoundTracker IV (Unknown/D.O.C.), D.O.C. SoundTracker VI (Unknown/D.O.C.)
-	ST_IX,               // D.O.C. SoundTracker IX (Unknown/D.O.C.)
-	MST1_00,             // Master Soundtracker 1.0 (Tip/The New Masters)
-	ST2_00,              // SoundTracker 2.0, 2.1, 2.2 (Unknown/D.O.C.)
+	UST1_00,              // Ultimate Soundtracker 1.0-1.21 (K. Obarski)
+	UST1_80,              // Ultimate Soundtracker 1.8-2.0 (K. Obarski)
+	ST2_00_Exterminator,  // SoundTracker 2.0 (The Exterminator), D.O.C. Sountracker II (Unknown/D.O.C.)
+	ST_III,               // Defjam Soundtracker III (Il Scuro/Defjam), Alpha Flight SoundTracker IV (Alpha Flight), D.O.C. SoundTracker IV (Unknown/D.O.C.), D.O.C. SoundTracker VI (Unknown/D.O.C.)
+	ST_IX,                // D.O.C. SoundTracker IX (Unknown/D.O.C.)
+	MST1_00,              // Master Soundtracker 1.0 (Tip/The New Masters)
+	ST2_00,               // SoundTracker 2.0, 2.1, 2.2 (Unknown/D.O.C.)
 };
 
 
@@ -1311,8 +1356,6 @@ struct M15FileHeaders
 
 MPT_BINARY_STRUCT(M15FileHeaders, 20 + 15 * 30 + 130)
 
-typedef uint8 M15PatternData[64][4][4];
-
 
 static bool ValidateHeader(const M15FileHeaders &fileHeaders)
 {
@@ -1338,9 +1381,9 @@ static bool ValidateHeader(const M15FileHeaders &fileHeaders)
 
 		// Sanity checks - invalid character count adjusted for ata.mod (MD5 937b79b54026fa73a1a4d3597c26eace, SHA1 3322ca62258adb9e0ae8e9afe6e0c29d39add874)
 		if(invalidChars > 48
-			|| sampleHeader.volume > 64
-			|| sampleHeader.finetune != 0
-			|| sampleHeader.length > 32768)
+		   || sampleHeader.volume > 64
+		   || sampleHeader.finetune != 0
+		   || sampleHeader.length > 32768)
 		{
 			return false;
 		}
@@ -1379,39 +1422,11 @@ static bool ValidateHeader(const M15FileHeaders &fileHeaders)
 }
 
 
-static uint32 CountIllegalM15PatternBytes(const M15PatternData &patternData)
-{
-	uint32 illegalBytes = 0;
-	for(uint8 row = 0; row < 64; ++row)
-	{
-		for(uint8 channel = 0; channel < 4; ++channel)
-		{
-			if(patternData[row][channel][0] & 0xF0u)
-			{
-				illegalBytes++;
-			}
-		}
-	}
-	return illegalBytes;
-}
-
-
 template <typename TFileReader>
 static bool ValidateFirstM15Pattern(TFileReader &file)
 {
-	M15PatternData patternData;
-	if(!file.ReadArray(patternData))
-	{
-		return false;
-	}
-	file.SkipBack(sizeof(patternData));
-	uint32 invalidBytes = CountIllegalM15PatternBytes(patternData);
-	// [threshold for all patterns combined] / [max patterns] * [margin, do not reject too much]
-	if(invalidBytes > 512 / 64 * 2)
-	{
-		return false;
-	}
-	return true;
+	// threshold is chosen as: [threshold for all patterns combined] / [max patterns] * [margin, do not reject too much]
+	return ValidateMODPatternData(file, 512 / 64 * 2, false);
 }
 
 
@@ -1426,7 +1441,7 @@ CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderM15(MemoryFileReader file, co
 	{
 		return ProbeFailure;
 	}
-	if(!file.CanRead(sizeof(M15PatternData)))
+	if(!file.CanRead(sizeof(MODPatternData)))
 	{
 		return ProbeWantMoreData;
 	}
@@ -1463,7 +1478,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 	InitializeGlobals(MOD_TYPE_MOD);
 	m_playBehaviour.reset(kMODOneShotLoops);
 	m_playBehaviour.set(kMODIgnorePanning);
-	m_playBehaviour.set(kMODSampleSwap);	// untested
+	m_playBehaviour.set(kMODSampleSwap);  // untested
 	m_nChannels = 4;
 
 	STVersions minVersion = UST1_00;
@@ -1480,7 +1495,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 
 		totalSampleLen += Samples[smp].nLength;
 
-		if(m_szNames[smp][0] && ((memcmp(m_szNames[smp], "st-", 3) && memcmp(m_szNames[smp], "ST-", 3)) || m_szNames[smp][5] != ':'))
+		if(m_szNames[smp][0] && ((memcmp(m_szNames[smp].buf, "st-", 3) && memcmp(m_szNames[smp].buf, "ST-", 3)) || m_szNames[smp][5] != ':'))
 		{
 			// Ultimate Soundtracker 1.8 and D.O.C. SoundTracker IX always have sample names containing disk names.
 			hasDiskNames = false;
@@ -1503,7 +1518,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 	file.ReadStruct(fileHeader);
 
 	ReadOrderFromArray(Order(), fileHeader.orderList);
-	PATTERNINDEX numPatterns = GetNumPatterns(file, Order(), fileHeader.numOrders, totalSampleLen, m_nChannels, false);
+	PATTERNINDEX numPatterns = GetNumPatterns(file, Order(), fileHeader.numOrders, totalSampleLen, m_nChannels, 0, true);
 
 	// Most likely just a file with lots of NULs at the start
 	if(fileHeader.restartPos == 0 && fileHeader.numOrders == 0 && numPatterns <= 1)
@@ -1526,7 +1541,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 	if(!memcmp(songname, "jjk55", 6))
 		fileHeader.restartPos = 0x78;
 	// Sample 7 in echoing.mod won't "loop" correctly if we don't convert the VBlank tempo.
-	m_nDefaultTempo.Set(fileHeader.restartPos * 25 / 24);
+	m_nDefaultTempo.Set(125);
 	if(fileHeader.restartPos != 0x78)
 	{
 		// Convert to CIA timing
@@ -1545,7 +1560,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 	m_nMaxPeriod = 856 * 4;
 	m_nSamplePreAmp = 64;
 	m_SongFlags.set(SONG_PT_MODE);
-	mpt::String::Read<mpt::String::spacePadded>(m_songName, songname);
+	m_songName = mpt::String::ReadBuf(mpt::String::spacePadded, songname);
 
 	// Setup channel pan positions and volume
 	SetupMODPanning();
@@ -1559,11 +1574,11 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 		bool patternInUse = std::find(Order().cbegin(), Order().cend(), pat) != Order().cend();
 		uint8 numDxx = 0;
 		uint8 emptyCmds = 0;
-		M15PatternData patternData;
+		MODPatternData patternData;
 		file.ReadArray(patternData);
 		if(patternInUse)
 		{
-			illegalBytes += CountIllegalM15PatternBytes(patternData);
+			illegalBytes += CountMalformedMODPatternData(patternData, false);
 			// Reject files that contain a lot of illegal pattern data.
 			// STK.the final remix (MD5 5ff13cdbd77211d1103be7051a7d89c9, SHA1 e94dba82a5da00a4758ba0c207eb17e3a89c3aa3)
 			// has one illegal byte, so we only reject after an arbitrary threshold has been passed.
@@ -1582,10 +1597,10 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 		{
 			for(CHANNELINDEX chn = 0; chn < 4; chn++)
 			{
-				const uint8 (&data)[4] = patternData[row][chn];
+				const auto &data = patternData[row][chn];
 				const uint8 eff = data[2] & 0x0F, param = data[3];
 				// Check for empty space between the last Dxx command and the beginning of another pattern
-				if(emptyCmds != 0 && !memcmp(data, "\0\0\0\0", 4))
+				if(emptyCmds != 0 && !memcmp(data.data(), "\0\0\0\0", 4))
 				{
 					emptyCmds++;
 					if(emptyCmds > 32)
@@ -1662,7 +1677,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 		Patterns.ResizeArray(numPatterns);
 	for(PATTERNINDEX pat = 0; pat < numPatterns; pat++)
 	{
-		M15PatternData patternData;
+		MODPatternData patternData;
 		file.ReadArray(patternData);
 
 		if(!(loadFlags & loadPatternData) || !Patterns.Insert(pat, 64))
@@ -1670,7 +1685,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 			continue;
 		}
 
-		uint8 autoSlide[4] = { 0, 0, 0, 0 };
+		uint8 autoSlide[4] = {0, 0, 0, 0};
 		for(ROWINDEX row = 0; row < 64; row++)
 		{
 			PatternRow rowBase = Patterns[pat].GetpModCommand(row, 0);
@@ -1767,7 +1782,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 		}
 	}
 
-	const MPT_UCHAR_TYPE *madeWithTracker = UL_("");
+	const mpt::uchar *madeWithTracker = UL_("");
 	switch(minVersion)
 	{
 	case UST1_00:
@@ -1796,7 +1811,7 @@ bool CSoundFile::ReadM15(FileReader &file, ModLoadingFlags loadFlags)
 	m_modFormat.formatName = U_("Soundtracker");
 	m_modFormat.type = U_("stk");
 	m_modFormat.madeWithTracker = madeWithTracker;
-	m_modFormat.charset = mpt::CharsetISO8859_1;
+	m_modFormat.charset = mpt::Charset::ISO8859_1;
 
 	// Reading samples
 	if(loadFlags & loadSampleData)
@@ -1845,8 +1860,7 @@ CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderICE(MemoryFileReader file, co
 	{
 		return ProbeFailure;
 	}
-	const uint8 numOrders = file.ReadUint8();
-	const uint8 numTracks = file.ReadUint8();
+	const auto [numOrders, numTracks] = file.ReadArray<uint8, 2>();
 	if(numOrders > 128)
 	{
 		return ProbeFailure;
@@ -1879,20 +1893,20 @@ bool CSoundFile::ReadICE(FileReader &file, ModLoadingFlags loadFlags)
 	InitializeGlobals(MOD_TYPE_MOD);
 	m_playBehaviour.reset(kMODOneShotLoops);
 	m_playBehaviour.set(kMODIgnorePanning);
-	m_playBehaviour.set(kMODSampleSwap);	// untested
+	m_playBehaviour.set(kMODSampleSwap);  // untested
 
 	if(IsMagic(magic, "MTN\0"))
 	{
 		m_modFormat.formatName = U_("MnemoTroN SoundTracker");
 		m_modFormat.type = U_("st26");
 		m_modFormat.madeWithTracker = U_("SoundTracker 2.6");
-		m_modFormat.charset = mpt::CharsetISO8859_1;
+		m_modFormat.charset = mpt::Charset::ISO8859_1;
 	} else if(IsMagic(magic, "IT10"))
 	{
 		m_modFormat.formatName = U_("Ice Tracker");
 		m_modFormat.type = U_("ice");
 		m_modFormat.madeWithTracker = U_("Ice Tracker 1.0 / 1.1");
-		m_modFormat.charset = mpt::CharsetISO8859_1;
+		m_modFormat.charset = mpt::Charset::ISO8859_1;
 	} else
 	{
 		return false;
@@ -1915,8 +1929,7 @@ bool CSoundFile::ReadICE(FileReader &file, ModLoadingFlags loadFlags)
 		return false;
 	}
 
-	const uint8 numOrders = file.ReadUint8();
-	const uint8 numTracks = file.ReadUint8();
+	const auto [numOrders, numTracks] = file.ReadArray<uint8, 2>();
 	if(numOrders > 128)
 	{
 		return false;
@@ -1945,14 +1958,14 @@ bool CSoundFile::ReadICE(FileReader &file, ModLoadingFlags loadFlags)
 	m_nMinPeriod = 14 * 4;
 	m_nMaxPeriod = 3424 * 4;
 	m_nSamplePreAmp = 64;
-	m_SongFlags.set(SONG_PT_MODE);
+	m_SongFlags.set(SONG_PT_MODE | SONG_IMPORTED);
 
 	// Setup channel pan positions and volume
 	SetupMODPanning();
 
 	// Reading patterns
 	Order().resize(numOrders);
-	uint8 speed[2] = { 0, 0 }, speedPos = 0;
+	uint8 speed[2] = {0, 0}, speedPos = 0;
 	Patterns.ResizeArray(numOrders);
 	for(PATTERNINDEX pat = 0; pat < numOrders; pat++)
 	{
@@ -1970,8 +1983,8 @@ bool CSoundFile::ReadICE(FileReader &file, ModLoadingFlags loadFlags)
 				ReadMODPatternEntry(file, *m);
 
 				if((m->command || m->param)
-					&& !(m->command == 0x0E && m->param >= 0x10)	// Exx only sets filter
-					&& !(m->command >= 0x05 && m->command <= 0x09))	// These don't exist in ST2.6
+				   && !(m->command == 0x0E && m->param >= 0x10)     // Exx only sets filter
+				   && !(m->command >= 0x05 && m->command <= 0x09))  // These don't exist in ST2.6
 				{
 					ConvertModCommand(*m);
 				} else
@@ -2008,7 +2021,8 @@ bool CSoundFile::ReadICE(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				Patterns[pat].WriteEffect(EffectWriter(CMD_SPEED, speed[speedPos - 1]).Row(row));
 				speedPos++;
-				if(speedPos == 3) speedPos = 1;
+				if(speedPos == 3)
+					speedPos = 1;
 			}
 		}
 	}
@@ -2126,7 +2140,7 @@ bool CSoundFile::ReadPT36(FileReader &file, ModLoadingFlags loadFlags)
 			chunk.Skip(4);
 			if(chunk.ReadMagic("PT") && iffHead.chunksize > 6)
 			{
-				chunk.ReadString<mpt::String::maybeNullTerminated>(version, mpt::CharsetISO8859_1, iffHead.chunksize - 6);
+				chunk.ReadString<mpt::String::maybeNullTerminated>(version, mpt::Charset::ISO8859_1, iffHead.chunksize - 6);
 			}
 			break;
 
@@ -2155,15 +2169,15 @@ bool CSoundFile::ReadPT36(FileReader &file, ModLoadingFlags loadFlags)
 		bool vblank = (info.flags & 0x100) == 0;
 		m_playBehaviour.set(kMODVBlankTiming, vblank);
 		if(info.volume != 0)
-			m_nSamplePreAmp = std::min<uint16>(64, info.volume);
+			m_nSamplePreAmp = std::min(uint16(64), static_cast<uint16>(info.volume));
 		if(info.tempo != 0 && !vblank)
 			m_nDefaultTempo.Set(info.tempo);
 
 		if(info.name[0])
-			mpt::String::Read<mpt::String::maybeNullTerminated>(m_songName, info.name);
+			m_songName = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, info.name);
 
 		if(IsInRange(info.dateMonth, 1, 12) && IsInRange(info.dateDay, 1, 31) && IsInRange(info.dateHour, 0, 23)
-			&& IsInRange(info.dateMinute, 0, 59) && IsInRange(info.dateSecond, 0, 59))
+		   && IsInRange(info.dateMinute, 0, 59) && IsInRange(info.dateSecond, 0, 59))
 		{
 			FileHistory mptHistory;
 			mptHistory.loadDate.tm_year = info.dateYear;
@@ -2182,7 +2196,7 @@ bool CSoundFile::ReadPT36(FileReader &file, ModLoadingFlags loadFlags)
 			std::string author;
 			commentChunk.ReadString<mpt::String::maybeNullTerminated>(author, 32);
 			if(author != "UNNAMED AUTHOR")
-				m_songArtist = mpt::ToUnicode(mpt::CharsetISO8859_1, author);
+				m_songArtist = mpt::ToUnicode(mpt::Charset::ISO8859_1, author);
 			if(!commentChunk.NoBytesLeft())
 			{
 				m_songMessage.ReadFixedLineLength(commentChunk, commentChunk.BytesLeft(), 40, 0);
@@ -2212,7 +2226,7 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 	// Write song title
 	{
 		char name[20];
-		mpt::String::Write<mpt::String::maybeNullTerminated>(name, m_songName);
+		mpt::String::WriteBuf(mpt::String::maybeNullTerminated, name) = m_songName;
 		mpt::IO::Write(f, name);
 	}
 
@@ -2225,11 +2239,11 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 		for(INSTRUMENTINDEX ins = 1; ins <= lastIns; ins++) if (Instruments[ins])
 		{
 			// Find some valid sample associated with this instrument.
-			for(size_t i = 0; i < CountOf(Instruments[ins]->Keyboard); i++)
+			for(auto smp : Instruments[ins]->Keyboard)
 			{
-				if(Instruments[ins]->Keyboard[i] > 0 && Instruments[ins]->Keyboard[i] <= GetNumSamples())
+				if(smp > 0 && smp <= GetNumSamples())
 				{
-					sampleSource[ins] = Instruments[ins]->Keyboard[i];
+					sampleSource[ins] = smp;
 					break;
 				}
 			}
@@ -2246,7 +2260,7 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 	for(SAMPLEINDEX smp = 1; smp <= 31; smp++)
 	{
 		MODSampleHeader sampleHeader;
-		mpt::String::Write<mpt::String::maybeNullTerminated>(sampleHeader.name, m_szNames[sampleSource[smp]]);
+		mpt::String::WriteBuf(mpt::String::maybeNullTerminated, sampleHeader.name) = m_szNames[sampleSource[smp]];
 		sampleLength[smp] = sampleHeader.ConvertToMOD(sampleSource[smp] <= GetNumSamples() ? GetSample(sampleSource[smp]) : ModSample(MOD_TYPE_MOD));
 		mpt::IO::Write(f, sampleHeader);
 	}
@@ -2326,7 +2340,7 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 			size_t eventByte = 0;
 			for(CHANNELINDEX chn = 0; chn < writeChannels; chn++, eventByte += 4)
 			{
-				ModCommand &m = rowBase[chn];
+				const ModCommand &m = rowBase[chn];
 				uint8 command = m.command, param = m.param;
 				ModSaveCommand(command, param, false, true);
 
@@ -2334,12 +2348,12 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 				{
 					// Maybe we can save some volume commands...
 					command = 0x0C;
-					param = MIN(m.vol, 64);
+					param = std::min(m.vol, uint8(64));
 				}
 
 				uint16 period = 0;
 				// Convert note to period
-				if(m.note >= 24 + NOTE_MIN && m.note < mpt::size(ProTrackerPeriodTable) + 24 + NOTE_MIN)
+				if(m.note >= 24 + NOTE_MIN && m.note < std::size(ProTrackerPeriodTable) + 24 + NOTE_MIN)
 				{
 					period = ProTrackerPeriodTable[m.note - 24 - NOTE_MIN];
 				}
@@ -2406,7 +2420,7 @@ bool CSoundFile::SaveMod(std::ostream &f) const
 	return true;
 }
 
-#endif // MODPLUG_NO_FILESAVE
+#endif  // MODPLUG_NO_FILESAVE
 
 
 OPENMPT_NAMESPACE_END
