@@ -59,7 +59,7 @@ struct STPSampleHeader
 	void ConvertToMPT(ModSample &mptSmp) const
 	{
 		mptSmp.nLength = length;
-		mptSmp.nVolume = 4u * std::min<uint16>(volume, 64);
+		mptSmp.nVolume = 4u * std::min(volume.get(), uint8(64));
 
 		mptSmp.nLoopStart = loopStart;
 		mptSmp.nLoopEnd = loopStart + loopLength;
@@ -188,7 +188,7 @@ static void ConvertLoopSequence(ModSample &smp, STPLoopList &loopList)
 
 		// update loop info based on position in edited sample
 		info.loopStart = start;
-		if(i > 0 && i <= mpt::size(newSmp.cues))
+		if(i > 0 && i <= std::size(newSmp.cues))
 		{
 			newSmp.cues[i - 1] = start;
 		}
@@ -258,7 +258,7 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 
 	m_modFormat.formatName = mpt::format(U_("Soundtracker Pro II v%1"))(fileHeader.version);
 	m_modFormat.type = U_("stp");
-	m_modFormat.charset = mpt::CharsetISO8859_1;
+	m_modFormat.charset = mpt::Charset::ISO8859_1;
 
 	m_nChannels = 4;
 	m_nSamples = 0;
@@ -306,12 +306,12 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 			std::string str;
 			// Read path
 			chunk.ReadNullString(str, 257);
-			mpt::String::Copy(mptSmp.filename, str);
+			mptSmp.filename = str;
 			// Ignore flags, they are all not relevant for us
 			chunk.Skip(1);
 			// Read filename / sample text
 			chunk.ReadNullString(str, 31);
-			mpt::String::Copy(m_szNames[actualSmp], str);
+			m_szNames[actualSmp] = str;
 			// Seek to even boundary
 			if(chunk.GetPosition() % 2u)
 				chunk.Skip(1);
@@ -435,13 +435,11 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 			{
 				ChannelMemory &chnMem = channelMemory[chn];
 				ModCommand &m = rowBase[chn];
-				uint8 data[4];
-				file.ReadArray(data);
+				const auto [instr, note, command, param] = file.ReadArray<uint8, 4>();
 
-				m.instr   = data[0];
-				m.note    = data[1];
-				m.command = data[2];
-				m.param   = data[3];
+				m.instr   = instr;
+				m.note    = note;
+				m.param   = param;
 
 				if(m.note)
 				{
@@ -453,10 +451,10 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 				// and auto global fine volside
 				uint8 swapped = (m.param >> 4) | (m.param << 4);
 
-				if((m.command & 0xF0) == 0xF0)
+				if((command & 0xF0) == 0xF0)
 				{
 					// 12-bit CIA tempo
-					uint16 ciaTempo = (static_cast<uint16>(m.command & 0x0F) << 8) | m.param;
+					uint16 ciaTempo = (static_cast<uint16>(command & 0x0F) << 8) | m.param;
 					if(ciaTempo)
 					{
 						m.param = mpt::saturate_round<ModCommand::PARAM>(ConvertTempo(ciaTempo).ToDouble());
@@ -465,11 +463,13 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 					{
 						m.command = CMD_NONE;
 					}
-				} else switch(m.command)
+				} else switch(command)
 				{
 				case 0x00: // arpeggio
 					if(m.param)
 						m.command = CMD_ARPEGGIO;
+					else
+						m.command = CMD_NONE;
 					break;
 
 				case 0x01: // portamento up
@@ -611,7 +611,7 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 						STPLoopList &loopList = loopInfo[m.instr - 1];
 
 						m.param--;
-						if(m.param < std::min(mpt::size(ModSample().cues), loopList.size()))
+						if(m.param < std::min(std::size(ModSample().cues), loopList.size()))
 						{
 							m.volcmd = VOLCMD_OFFSET;
 							m.vol = m.param;
@@ -629,7 +629,7 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 						m.param--;
 						if(m.param < loopList.size())
 						{
-							if(!loopList[m.param].looped && m_nSamples < MAX_SAMPLES - 1)
+							if(!loopList[m.param].looped && CanAddMoreSamples())
 								loopList[m.param].looped = ++m_nSamples;
 							m.instr = static_cast<ModCommand::INSTR>(loopList[m.param].looped);
 						}
@@ -644,13 +644,13 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 						STPLoopList &loopList = loopInfo[m.instr - 1];
 
 						m.param--;
-						if(m.param < std::min(mpt::size(ModSample().cues), loopList.size()))
+						if(m.param < std::min(std::size(ModSample().cues), loopList.size()))
 						{
 							m.volcmd = VOLCMD_OFFSET;
 							m.vol = m.param;
 						}
 						// switch to non-looped version of sample and create it if needed
-						if(!nonLooped[m.instr - 1] && m_nSamples < MAX_SAMPLES - 1)
+						if(!nonLooped[m.instr - 1] && CanAddMoreSamples())
 							nonLooped[m.instr - 1] = ++m_nSamples;
 						m.instr = static_cast<ModCommand::INSTR>(nonLooped[m.instr - 1]);
 					}
@@ -666,7 +666,7 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 						m.param--;
 						if(m.param < loopList.size())
 						{
-							if(!loopList[m.param].nonLooped && m_nSamples < MAX_SAMPLES-1)
+							if(!loopList[m.param].nonLooped && CanAddMoreSamples())
 								loopList[m.param].nonLooped = ++m_nSamples;
 							m.instr = static_cast<ModCommand::INSTR>(loopList[m.param].nonLooped);
 						}
