@@ -66,7 +66,7 @@ void ModSample::Convert(MODTYPE fromType, MODTYPE toType)
 			uFlags.set(CHN_PINGPONGLOOP, uFlags[CHN_PINGPONGSUSTAIN]);
 		}
 		nSustainStart = nSustainEnd = 0;
-		uFlags.reset(CHN_SUSTAINLOOP|CHN_PINGPONGSUSTAIN);
+		uFlags.reset(CHN_SUSTAINLOOP | CHN_PINGPONGSUSTAIN);
 	}
 
 	// All XM samples have default panning, and XM's autovibrato settings are rather limited.
@@ -125,6 +125,7 @@ void ModSample::Convert(MODTYPE fromType, MODTYPE toType)
 // Initialize sample slot with default values.
 void ModSample::Initialize(MODTYPE type)
 {
+	FreeSample();
 	nLength = 0;
 	nLoopStart = nLoopEnd = 0;
 	nSustainStart = nSustainEnd = 0;
@@ -144,7 +145,7 @@ void ModSample::Initialize(MODTYPE type)
 	nVibDepth = 0;
 	nVibRate = 0;
 	rootNote = 0;
-	filename[0] = '\0';
+	filename = "";
 
 	SetDefaultCuePoints();
 }
@@ -189,7 +190,7 @@ void *ModSample::AllocateSample(SmpLength numFrames, size_t bytesPerSample)
 
 	if(allocSize != 0)
 	{
-		char *p = new (std::nothrow) char[allocSize];
+		char *p = new(std::nothrow) char[allocSize];
 		if(p != nullptr)
 		{
 			memset(p, 0, allocSize);
@@ -209,7 +210,7 @@ size_t ModSample::GetRealSampleBufferSize(SmpLength numSamples, size_t bytesPerS
 	// * 2x InterpolationMaxLookahead before the loop point (because we start at InterpolationMaxLookahead before the loop point and will look backwards from there as well)
 	// * 2x InterpolationMaxLookahead after the loop point (for wrap-around)
 	// * 4x InterpolationMaxLookahead for the sustain loop (same as the two points above)
-	
+
 	const SmpLength maxSize = Util::MaxValueOfType(numSamples);
 	const SmpLength lookaheadBufferSize = (MaxSamplingPointSize + 1 + 4 + 4) * InterpolationMaxLookahead;
 
@@ -239,7 +240,7 @@ void ModSample::FreeSample(void *samplePtr)
 {
 	if(samplePtr)
 	{
-		delete[] (((char *)samplePtr) - (InterpolationMaxLookahead * MaxSamplingPointSize));
+		delete[](((char *)samplePtr) - (InterpolationMaxLookahead * MaxSamplingPointSize));
 	}
 }
 
@@ -282,9 +283,144 @@ void ModSample::SetSustainLoop(SmpLength start, SmpLength end, bool enable, bool
 }
 
 
+namespace  // Unnamed namespace for local implementation functions.
+{
+
+template <typename T>
+class PrecomputeLoop
+{
+protected:
+	T *target;
+	const T *sampleData;
+	SmpLength loopEnd;
+	int numChannels;
+	bool pingpong;
+	bool ITPingPongMode;
+
+public:
+	PrecomputeLoop(T *target, const T *sampleData, SmpLength loopEnd, int numChannels, bool pingpong, bool ITPingPongMode)
+	    : target(target), sampleData(sampleData), loopEnd(loopEnd), numChannels(numChannels), pingpong(pingpong), ITPingPongMode(ITPingPongMode)
+	{
+		if(loopEnd > 0)
+		{
+			CopyLoop(true);
+			CopyLoop(false);
+		}
+	}
+
+	void CopyLoop(bool direction) const
+	{
+		// Direction: true = start reading and writing forward, false = start reading and writing backward (write direction never changes)
+		const int numSamples = 2 * InterpolationMaxLookahead + (direction ? 1 : 0);  // Loop point is included in forward loop expansion
+		T *dest = target + numChannels * (2 * InterpolationMaxLookahead - 1);        // Write buffer offset
+		SmpLength readPosition = loopEnd - 1;
+		const int writeIncrement = direction ? 1 : -1;
+		int readIncrement = writeIncrement;
+
+		for(int i = 0; i < numSamples; i++)
+		{
+			// Copy sample over to lookahead buffer
+			for(int c = 0; c < numChannels; c++)
+			{
+				dest[c] = sampleData[readPosition * numChannels + c];
+			}
+			dest += writeIncrement * numChannels;
+
+			if(readPosition == loopEnd - 1 && readIncrement > 0)
+			{
+				// Reached end of loop while going forward
+				if(pingpong)
+				{
+					readIncrement = -1;
+					if(ITPingPongMode && readPosition > 0)
+					{
+						readPosition--;
+					}
+				} else
+				{
+					readPosition = 0;
+				}
+			} else if(readPosition == 0 && readIncrement < 0)
+			{
+				// Reached start of loop while going backward
+				if(pingpong)
+				{
+					readIncrement = 1;
+				} else
+				{
+					readPosition = loopEnd - 1;
+				}
+			} else
+			{
+				readPosition += readIncrement;
+			}
+		}
+	}
+};
+
+
+template <typename T>
+void PrecomputeLoopsImpl(ModSample &smp, const CSoundFile &sndFile)
+{
+	const int numChannels = smp.GetNumChannels();
+	const int copySamples = numChannels * InterpolationMaxLookahead;
+
+	T *sampleData = static_cast<T *>(smp.samplev());
+	T *afterSampleStart = sampleData + smp.nLength * numChannels;
+	T *loopLookAheadStart = afterSampleStart + copySamples;
+	T *sustainLookAheadStart = loopLookAheadStart + 4 * copySamples;
+
+	// Hold sample on the same level as the last sampling point at the end to prevent extra pops with interpolation.
+	// Do the same at the sample start, too.
+	for(int i = 0; i < (int)InterpolationMaxLookahead; i++)
+	{
+		for(int c = 0; c < numChannels; c++)
+		{
+			afterSampleStart[i * numChannels + c] = afterSampleStart[-numChannels + c];
+			sampleData[-(i + 1) * numChannels + c] = sampleData[c];
+		}
+	}
+
+	if(smp.uFlags[CHN_LOOP])
+	{
+		PrecomputeLoop<T>(loopLookAheadStart,
+			sampleData + smp.nLoopStart * numChannels,
+			smp.nLoopEnd - smp.nLoopStart,
+			numChannels,
+			smp.uFlags[CHN_PINGPONGLOOP],
+			sndFile.m_playBehaviour[kITPingPongMode]);
+	}
+	if(smp.uFlags[CHN_SUSTAINLOOP])
+	{
+		PrecomputeLoop<T>(sustainLookAheadStart,
+			sampleData + smp.nSustainStart * numChannels,
+			smp.nSustainEnd - smp.nSustainStart,
+			numChannels,
+			smp.uFlags[CHN_PINGPONGSUSTAIN],
+			sndFile.m_playBehaviour[kITPingPongMode]);
+	}
+}
+
+}  // unnamed namespace
+
+
 void ModSample::PrecomputeLoops(CSoundFile &sndFile, bool updateChannels)
 {
-	ctrlSmp::PrecomputeLoops(*this, sndFile, updateChannels);
+	if(!HasSampleData())
+		return;
+
+	SanitizeLoops();
+
+	// Update channels with possibly changed loop values
+	if(updateChannels)
+	{
+		ctrlSmp::UpdateLoopPoints(*this, sndFile);
+	}
+
+	if(GetElementarySampleSize() == 2)
+		PrecomputeLoopsImpl<int16>(*this, sndFile);
+	else if(GetElementarySampleSize() == 1)
+		PrecomputeLoopsImpl<int8>(*this, sndFile);
 }
 
 
@@ -321,20 +457,21 @@ void ModSample::TransposeToFrequency()
 }
 
 
-// Return tranpose.finetune as 25.7 fixed point value.
-int ModSample::FrequencyToTranspose(uint32 freq)
+// Return a pair of {tranpose, finetune}
+std::pair<int8, int8> ModSample::FrequencyToTranspose(uint32 freq)
 {
-	return mpt::saturate_round<int>(std::log(freq * (1.0 / 8363.0)) * (12.0 * 128.0 * (1.0 / M_LN2)));
+	if(!freq)
+		return {};
+
+	const auto f2t = mpt::saturate_round<int32>(std::log(freq * (1.0 / 8363.0)) * (12.0 * 128.0 * (1.0 / M_LN2)));
+	const auto fine = std::div(Clamp(f2t, -16384, 16383), int32(128));
+	return {static_cast<int8>(fine.quot), static_cast<int8>(fine.rem)};
 }
 
 
 void ModSample::FrequencyToTranspose()
 {
-	int f2t = 0;
-	if(nC5Speed)
-		f2t = FrequencyToTranspose(nC5Speed);
-	RelativeTone = static_cast<int8>(f2t >> 7);
-	nFineTune = static_cast<int8>(f2t & 0x7F);
+	std::tie(RelativeTone, nFineTune) = FrequencyToTranspose(nC5Speed);
 }
 
 
@@ -352,7 +489,8 @@ bool ModSample::HasCustomCuePoints() const
 	{
 		for(SmpLength i = 0; i < CountOf(cues); i++)
 		{
-			if(cues[i] != (i + 1) << 11) return true;
+			if(cues[i] != (i + 1) << 11)
+				return true;
 		}
 	}
 	return false;
