@@ -74,14 +74,14 @@ public:
 	void Mix(int32 *buffer, size_t count, uint32 volumeFactorQ16);
 
 	void NoteOff(CHANNELINDEX c);
-	void NoteCut(CHANNELINDEX c);
+	void NoteCut(CHANNELINDEX c, bool unassign = true);
 	void Frequency(CHANNELINDEX c, uint32 milliHertz, bool keyOff, bool beatingOscillators);
 	void Volume(CHANNELINDEX c, uint8 vol, bool applyToModulator);
 	int8 Pan(CHANNELINDEX c, int32 pan);
 	void Patch(CHANNELINDEX c, const OPLPatch &patch);
-	void Reset();
-	bool IsActive(CHANNELINDEX c) { return GetVoice(c) != OPL_CHANNEL_INVALID; }
+	bool IsActive(CHANNELINDEX c) const { return GetVoice(c) != OPL_CHANNEL_INVALID; }
 	void MoveChannel(CHANNELINDEX from, CHANNELINDEX to);
+	void Reset();
 
 protected:
 	static uint16 ChannelToRegister(uint8 oplCh);
@@ -92,7 +92,9 @@ protected:
 
 	enum
 	{
-		OPL_CHANNELS = 18,	// 9 for OPL2 or 18 for OPL3
+		OPL_CHANNELS = 18,       // 9 for OPL2 or 18 for OPL3
+		OPL_CHANNEL_CUT = 0x80,  // Indicates that the channel has been cut and used as a hint to re-use the channel for the same tracker channel if possible
+		OPL_CHANNEL_MASK = 0x7F,
 		OPL_CHANNEL_INVALID = 0xFF,
 		OPL_BASERATE = 49716,
 	};
